=== modified file 'HACKING'
--- HACKING	2006-06-06 22:59:58 +0000
+++ HACKING	2006-06-19 19:48:46 +0000
@@ -290,6 +290,10 @@
 
   ./bzr selftest -v blackbox
 
+To skip a particular test (or set of tests), you need to use a negative
+match, like so::
+  ./bzr selftest '^(?!.*blackbox)'  
+
 
 Errors and exceptions
 =====================

=== modified file 'NEWS'
--- NEWS	2006-06-13 16:26:09 +0000
+++ NEWS	2006-06-20 00:54:32 +0000
@@ -51,7 +51,19 @@
    * 'bzr log --line' shows the revision number, and uses only the
      first line of the log message (#5162, Alexander Belchenko;
      Matthieu Moy)
+
+   * 'bzr status' has had the --all option removed. The 'bzr ls' command
+     should be used to retrieve all versioned files. (Robert Collins)
+
+   * 'bzr bundle OTHER/BRANCH' will create a bundle which can be sent
+     over email, and applied on the other end, while maintaining ancestry.
+     This bundle can be applied with either 'bzr merge' or 'bzr pull',
+     the same way you would apply another branch.
+     (John Arbash Meinel, Aaron Bentley)
   
+   * 'branches.conf' has been changed to 'locations.conf', since it can apply
+     to more locations than just branch locations.
+     (Aaron Bentley)
 
   BUG FIXES:
 
@@ -62,6 +74,9 @@
     * Fix unnecessary requirement of sign-my-commits that it be run from
       a working directory.  (Martin Pool, Robert Collins)
 
+    * 'bzr push location' will only remember the push location if it succeeds
+      in connecting to the remote location. (#49742, John Arbash Meinel)
+
   INTERNALS:
 
     * Combine the ignore rules into a single regex rather than looping over
@@ -87,6 +102,14 @@
       that do not use '.bzr' to store their data - i.e. '.svn', '.hg' etc.
       (Robert Collins, Jelmer Vernooij).
 
+    * bzrlib.diff.external_diff can be redirected to any file-like object.
+      Uses subprocess instead of spawnvp.
+      (#4047, #48914, James Henstridge, John Arbash Meinel)
+
+    * New command line option '--profile-imports', which will install a custom
+      importer to log time to import modules and regex compilation time to 
+      sys.stderr (John Arbash Meinel)
+
 bzr 0.8.2  2006-05-17
   
   BUG FIXES:

=== modified file 'bzr'
--- bzr	2006-05-17 07:09:13 +0000
+++ bzr	2006-06-19 14:40:19 +0000
@@ -16,7 +16,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Bazaar-NG -- a free distributed version-control tool"""
-import os, sys
+
+import os
+import sys
+
+
 if __doc__ is None:
     print "bzr does not support python -OO."
     sys.exit(2)
@@ -44,6 +48,16 @@
 if hasattr(os, "unsetenv"):
     os.unsetenv(REINVOKE)
 
+
+
+profiling = False
+if '--profile-imports' in sys.argv:
+    sys.argv.remove('--profile-imports')
+    import profile_imports
+    profile_imports.install()
+    profiling = True
+
+
 try:
     import bzrlib
     import bzrlib.commands
@@ -63,6 +77,10 @@
 
 if __name__ == '__main__':
     bzrlib.trace.enable_default_logging()
-    sys.exit(bzrlib.commands.main(sys.argv))
+    exit_val = bzrlib.commands.main(sys.argv)
+
+    if profiling:
+        profile_imports.log_stack_info(sys.stderr)
+    sys.exit(exit_val)
 else:
     pass    # should this give an error? - it can't be used as a lib

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-06-20 03:57:11 +0000
+++ bzrlib/benchmarks/__init__.py	2006-06-20 05:32:16 +0000
@@ -19,6 +19,7 @@
 
 import bzrlib
 from bzrlib.tests.TestUtil import TestLoader
+from bzrlib.bzrdir import BzrDir
 from bzrlib.tests.blackbox import ExternalBase
 
 class Benchmark(ExternalBase):
@@ -51,6 +52,28 @@
                     files.extend([prefix + str(foo) for foo in range(20)])
         self.build_tree(files)
 
+    def make_many_commit_tree(self, directory_name='.'):
+        """Create a tree with an egregious number of commits.
+        
+        No files change are included.
+        """
+        tree = BzrDir.create_standalone_workingtree(directory_name)
+        tree.lock_write()
+        tree.branch.lock_write()
+        tree.branch.repository.lock_write()
+        try:
+            for i in xrange(1000):
+                tree.commit('no-changes commit %d' % i)
+        finally:
+            try:
+                try:
+                    tree.branch.repository.unlock()
+                finally:
+                    tree.branch.unlock()
+            finally:
+                tree.unlock()
+        return tree
+
 
 def test_suite():
     """Build and return a TestSuite which contains benchmark tests only."""
@@ -60,6 +83,7 @@
                    'bzrlib.benchmarks.bench_checkout',
                    'bzrlib.benchmarks.bench_commit',
                    'bzrlib.benchmarks.bench_inventory',
+                   'bzrlib.benchmarks.bench_log',
                    'bzrlib.benchmarks.bench_osutils',
                    'bzrlib.benchmarks.bench_rocks',
                    'bzrlib.benchmarks.bench_status',

=== added file 'bzrlib/benchmarks/bench_log.py'
--- bzrlib/benchmarks/bench_log.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_log.py	2006-06-17 18:37:30 +0000
@@ -0,0 +1,83 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for tree transform performance"""
+
+import os
+import sys
+
+from bzrlib.benchmarks import Benchmark
+from bzrlib.log import log_formatter, show_log
+from bzrlib.osutils import pathjoin
+from cStringIO import StringIO
+from bzrlib.transform import TreeTransform
+from bzrlib.workingtree import WorkingTree
+
+class LinesDone(Exception):
+    pass
+
+class LineConsumer(object):
+
+    def __init__(self, required_lines):
+        self.required_lines = required_lines
+
+    def write(self, text):
+        self.required_lines -= text.count('\n')
+        if self.required_lines < 0:
+            raise LinesDone()
+        
+
+class LogBenchmark(Benchmark):
+
+    def test_log(self):
+        """Run log in a many-commit tree.""" 
+        tree = self.make_many_commit_tree()
+        lf = log_formatter('long', to_file=StringIO())
+        self.time(show_log, tree.branch, lf, direction='reverse')
+
+    def test_log_screenful(self):
+        """Simulate log --long|less"""
+        self.screenful_tester('long')
+
+    def test_log_screenful_line(self):
+        """Simulate log --line|less"""
+        self.screenful_tester('line')
+
+    def test_log_screenful_short(self):
+        """Simulate log --short|less"""
+        self.screenful_tester('short')
+
+    def screenful_tester(self, formatter):
+        """Run show_log, but stop after 25 lines are generated"""
+        tree = self.make_many_commit_tree()
+        def log_screenful():
+            lf = log_formatter(formatter, to_file=LineConsumer(25))
+            try:
+                show_log(tree.branch, lf, direction='reverse')
+            except LinesDone:
+                pass
+            else:
+                raise Exception, "LinesDone not raised"
+        self.time(log_screenful)
+
+    def test_cmd_log(self):
+        """Test execution of the log command.""" 
+        tree = self.make_many_commit_tree()
+        self.time(self.run_bzr, 'log', '-r', '-4..')
+
+    def test_cmd_log_subprocess(self):
+        """Text startup and execution of the log command.""" 
+        tree = self.make_many_commit_tree()
+        self.time(self.run_bzr_subprocess, 'log', '-r', '-4..')

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-06-20 04:45:30 +0000
+++ bzrlib/branch.py	2006-06-20 05:32:16 +0000
@@ -27,12 +27,14 @@
         urlutils
 from bzrlib.config import TreeConfig
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.delta import compare_trees
-from bzrlib.errors import (InvalidRevisionNumber,
-                           NotBranchError,
-                           DivergedBranches,
-                           NoSuchFile,
-                           NoWorkingTree)
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, BzrCheckError, DivergedBranches, 
+                           HistoryMissing, InvalidRevisionId, 
+                           InvalidRevisionNumber, LockError, NoSuchFile, 
+                           NoSuchRevision, NoWorkingTree, NotVersionedError,
+                           NotBranchError, UninitializableFormat, 
+                           UnlistableStore, UnlistableBranch, 
+                           )
 from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.symbol_versioning import (deprecated_function,
                                       deprecated_method,
@@ -255,6 +257,18 @@
         """
         return None
 
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+        return self.repository.get_revision_delta(rh[revno-1])
+
     def get_root_id(self):
         """Return the id of this branches root"""
         raise NotImplementedError('get_root_id is abstract')
@@ -298,32 +312,6 @@
         
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
-
-        >>> from bzrlib.workingtree import WorkingTree
-        >>> bzrlib.trace.silent = True
-        >>> d1 = bzrdir.ScratchDir()
-        >>> br1 = d1.open_branch()
-        >>> wt1 = d1.open_workingtree()
-        >>> d2 = bzrdir.ScratchDir()
-        >>> br2 = d2.open_branch()
-        >>> wt2 = d2.open_workingtree()
-        >>> br1.missing_revisions(br2)
-        []
-        >>> wt2.commit("lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-1']
-        >>> br2.missing_revisions(br1)
-        []
-        >>> wt1.commit("lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        []
-        >>> wt2.commit("lala!", rev_id="REVISION-ID-2A")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-2A']
-        >>> wt1.commit("lala!", rev_id="REVISION-ID-2B")
-        >>> br1.missing_revisions(br2)
-        Traceback (most recent call last):
-        DivergedBranches: These branches have diverged.  Try merge.
         """
         self_history = self.revision_history()
         self_len = len(self_history)
@@ -600,7 +588,7 @@
         except NoSuchFile:
             raise NotBranchError(path=transport.base)
         except KeyError:
-            raise errors.UnknownFormatError(format_string)
+            raise errors.UnknownFormatError(format=format_string)
 
     @classmethod
     def get_default_format(klass):
@@ -900,11 +888,7 @@
                  stacklevel=2)
             if (not relax_version_check
                 and not self._format.is_supported()):
-                raise errors.UnsupportedFormatError(
-                        'sorry, branch format %r not supported' % self._format,
-                        ['use a different bzr version',
-                         'or remove the .bzr directory'
-                         ' and "bzr init" again'])
+                raise errors.UnsupportedFormatError(format=fmt)
         if deprecated_passed(transport):
             warn("BzrBranch.__init__(transport=XXX...): The transport "
                  "parameter is deprecated as of bzr 0.8. "
@@ -1049,26 +1033,6 @@
             # not really an object yet, and the transaction is for objects.
             # transaction.register_clean(history)
 
-    def get_revision_delta(self, revno):
-        """Return the delta for one revision.
-
-        The delta is relative to its mainline predecessor, or the
-        empty tree for revision 1.
-        """
-        assert isinstance(revno, int)
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
-            raise InvalidRevisionNumber(revno)
-
-        # revno is 1-based; list is 0-based
-
-        new_tree = self.repository.revision_tree(rh[revno-1])
-        if revno == 1:
-            old_tree = tree.EmptyTree()
-        else:
-            old_tree = self.repository.revision_tree(rh[revno-2])
-        return compare_trees(old_tree, new_tree)
-
     @needs_read_lock
     def revision_history(self):
         """See Branch.revision_history."""
@@ -1132,6 +1096,7 @@
     @deprecated_method(zero_eight)
     def working_tree(self):
         """Create a Working tree object for this branch."""
+
         from bzrlib.transport.local import LocalTransport
         if (self.base.find('://') != -1 or 
             not isinstance(self._transport, LocalTransport)):
@@ -1158,14 +1123,19 @@
 
     def get_parent(self):
         """See Branch.get_parent."""
+
         _locs = ['parent', 'pull', 'x-pull']
         assert self.base[-1] == '/'
         for l in _locs:
             try:
-                return urlutils.join(self.base[:-1],
-                            self.control_files.get(l).read().strip('\n'))
+                parent = self.control_files.get(l).read().strip('\n')
             except NoSuchFile:
-                pass
+                continue
+            # This is an old-format absolute path to a local branch
+            # turn it into a url
+            if parent.startswith('/'):
+                parent = urlutils.local_path_to_url(parent.decode('utf8'))
+            return urlutils.join(self.base[:-1], parent)
         return None
 
     def get_push_location(self):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-06-20 04:45:30 +0000
+++ bzrlib/builtins.py	2006-06-20 05:32:16 +0000
@@ -24,10 +24,10 @@
 
 import bzrlib
 from bzrlib.branch import Branch, BranchReferenceFormat
-from bzrlib import (branch, bzrdir, errors, osutils, ui, config,
+from bzrlib import (bundle, branch, bzrdir, errors, osutils, ui, config,
     repository, log)
 from bzrlib.bundle.read_bundle import BundleReader
-from bzrlib.bundle.apply_bundle import merge_bundle
+from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.commands import Command, display_command
 from bzrlib.errors import (BzrError, BzrCheckError, BzrCommandError, 
                            NotBranchError, DivergedBranches, NotConflicted,
@@ -126,10 +126,6 @@
     modified
         Text has changed since the previous revision.
 
-    unchanged
-        Nothing about this file has changed since the previous revision.
-        Only shown with --all.
-
     unknown
         Not versioned and not matching an ignore pattern.
 
@@ -148,18 +144,18 @@
     # TODO: --no-recurse, --recurse options
     
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
+    takes_options = ['show-ids', 'revision']
     aliases = ['st', 'stat']
 
     encoding_type = 'replace'
     
     @display_command
-    def run(self, all=False, show_ids=False, file_list=None, revision=None):
+    def run(self, show_ids=False, file_list=None, revision=None):
         from bzrlib.status import show_tree_status
 
         tree, file_list = tree_files(file_list)
             
-        show_tree_status(tree, show_unchanged=all, show_ids=show_ids,
+        show_tree_status(tree, show_ids=show_ids,
                          specific_files=file_list, revision=revision,
                          to_file=self.outf)
 
@@ -413,7 +409,8 @@
 
     If there is no default location set, the first pull will set it.  After
     that, you can omit the location to use the default.  To change the
-    default, use --remember.
+    default, use --remember. The value will only be saved if the remote
+    location can be accessed.
     """
 
     takes_options = ['remember', 'overwrite', 'revision', 'verbose']
@@ -428,6 +425,14 @@
         except NoWorkingTree:
             tree_to = None
             branch_to = Branch.open_containing(u'.')[0]
+
+        reader = None
+        if location is not None:
+            try:
+                reader = bundle.read_bundle_from_url(location)
+            except NotABundle:
+                pass # Continue on considering this url a Branch
+
         stored_loc = branch_to.get_parent()
         if location is None:
             if stored_loc is None:
@@ -438,13 +443,20 @@
                 self.outf.write("Using saved location: %s\n" % display_url)
                 location = stored_loc
 
-        branch_from = Branch.open(location)
-
-        if branch_to.get_parent() is None or remember:
-            branch_to.set_parent(branch_from.base)
-
+
+        if reader is not None:
+            install_bundle(branch_to.repository, reader)
+            branch_from = branch_to
+        else:
+            branch_from = Branch.open(location)
+
+            if branch_to.get_parent() is None or remember:
+                branch_to.set_parent(branch_from.base)
+
+        rev_id = None
         if revision is None:
-            rev_id = None
+            if reader is not None:
+                rev_id = reader.info.target
         elif len(revision) == 1:
             rev_id = revision[0].in_history(branch_from).rev_id
         else:
@@ -488,7 +500,8 @@
 
     If there is no default push location set, the first push will set it.
     After that, you can omit the location to use the default.  To change the
-    default, use --remember.
+    default, use --remember. The value will only be saved if the remote
+    location can be accessed.
     """
 
     takes_options = ['remember', 'overwrite', 'verbose',
@@ -512,13 +525,11 @@
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
-                self.outf.write("Using saved location: %s" % display_url)
+                self.outf.write("Using saved location: %s\n" % display_url)
                 location = stored_loc
 
         transport = get_transport(location)
         location_url = transport.base
-        if br_from.get_push_location() is None or remember:
-            br_from.set_push_location(location_url)
 
         old_rh = []
         try:
@@ -554,7 +565,14 @@
                 revision_id=br_from.last_revision())
             br_to = dir_to.open_branch()
             count = len(br_to.revision_history())
+            # We successfully created the target, remember it
+            if br_from.get_push_location() is None or remember:
+                br_from.set_push_location(br_to.base)
         else:
+            # We were able to connect to the remote location, so remember it
+            # we don't need to successfully push because of possible divergence.
+            if br_from.get_push_location() is None or remember:
+                br_from.set_push_location(br_to.base)
             old_rh = br_to.revision_history()
             try:
                 try:
@@ -922,27 +940,37 @@
 
 
 class cmd_revision_history(Command):
-    """Display list of revision ids on this branch."""
+    """Display the list of revision ids on a branch."""
+    takes_args = ['location?']
+
     hidden = True
 
     @display_command
-    def run(self):
-        branch = WorkingTree.open_containing(u'.')[0].branch
-        for patchid in branch.revision_history():
-            self.outf.write(patchid)
+    def run(self, location="."):
+        branch = Branch.open_containing(location)[0]
+        for revid in branch.revision_history():
+            self.outf.write(revid)
             self.outf.write('\n')
 
 
 class cmd_ancestry(Command):
     """List all revisions merged into this branch."""
+    takes_args = ['location?']
+
     hidden = True
 
     @display_command
-    def run(self):
-        tree = WorkingTree.open_containing(u'.')[0]
-        b = tree.branch
-        # FIXME. should be tree.last_revision
-        revision_ids = b.repository.get_ancestry(b.last_revision())
+    def run(self, location="."):
+        try:
+            wt = WorkingTree.open_containing(location)[0]
+        except errors.NoWorkingTree:
+            b = Branch.open(location)
+            last_revision = b.last_revision()
+        else:
+            b = wt.branch
+            last_revision = wt.last_revision()
+
+        revision_ids = b.repository.get_ancestry(last_revision)
         assert revision_ids[0] == None
         revision_ids.pop(0)
         for revision_id in revision_ids:
@@ -1993,7 +2021,8 @@
 
     If there is no default branch set, the first merge will set it. After
     that, you can omit the branch to use the default.  To change the
-    default, use --remember.
+    default, use --remember. The value will only be saved if the remote
+    location can be accessed.
 
     Examples:
 
@@ -2028,24 +2057,18 @@
 
         tree = WorkingTree.open_containing(u'.')[0]
 
-        try:
-            if branch is not None:
-                reader = BundleReader(file(branch, 'rb'))
-            else:
-                reader = None
-        except IOError, e:
-            if e.errno not in (errno.ENOENT, errno.EISDIR):
-                raise
-            reader = None
-        except NotABundle:
-            reader = None
-        if reader is not None:
-            conflicts = merge_bundle(reader, tree, not force, merge_type,
-                                        reprocess, show_base)
-            if conflicts == 0:
-                return 0
-            else:
-                return 1
+        if branch is not None:
+            try:
+                reader = bundle.read_bundle_from_url(branch)
+            except NotABundle:
+                pass # Continue on considering this url a Branch
+            else:
+                conflicts = merge_bundle(reader, tree, not force, merge_type,
+                                            reprocess, show_base)
+                if conflicts == 0:
+                    return 0
+                else:
+                    return 1
 
         branch = self._get_remembered_parent(tree, branch, 'Merging from')
 
@@ -2392,11 +2415,16 @@
 
 class cmd_testament(Command):
     """Show testament (signing-form) of a revision."""
-    takes_options = ['revision', 'long']
+    takes_options = ['revision', 'long', 
+                     Option('strict', help='Produce a strict testament')]
     takes_args = ['branch?']
     @display_command
-    def run(self, branch=u'.', revision=None, long=False):
-        from bzrlib.testament import Testament
+    def run(self, branch=u'.', revision=None, long=False, strict=False):
+        from bzrlib.testament import Testament, StrictTestament
+        if strict is True:
+            testament_class = StrictTestament
+        else:
+            testament_class = Testament
         b = WorkingTree.open_containing(branch)[0].branch
         b.lock_read()
         try:
@@ -2404,7 +2432,7 @@
                 rev_id = b.last_revision()
             else:
                 rev_id = revision[0].in_history(b).rev_id
-            t = Testament.from_revision(b.repository, rev_id)
+            t = testament_class.from_revision(b.repository, rev_id)
             if long:
                 sys.stdout.writelines(t.as_text_lines())
             else:

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2006-05-17 16:15:13 +0000
+++ bzrlib/bundle/__init__.py	2006-06-20 01:04:14 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import bzrlib.errors as errors
+import bzrlib.urlutils
+import bzrlib.transport
+import bzrlib.bundle.read_bundle
+
+
+def read_bundle_from_url(url):
+    """Read a bundle from a given URL.
+
+    :return: A BundleReader, may raise NotABundle if the target 
+            is not a proper bundle.
+    """
+    url, filename = bzrlib.urlutils.split(url, exclude_trailing_slash=False)
+    if not filename:
+        # A path to a directory was passed in
+        # definitely not a bundle
+        raise errors.NotABundle('A directory cannot be a bundle')
+
+    # All of this must be in the try/except
+    # Some transports cannot detect that we are trying to read a
+    # directory until we actually issue read() on the handle.
+    try:
+        t = bzrlib.transport.get_transport(url)
+        f = t.get(filename)
+        return bzrlib.bundle.read_bundle.BundleReader(f)
+    except (errors.TransportError, errors.PathError), e:
+        raise errors.NotABundle(str(e))
+

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2006-06-07 16:46:22 +0000
+++ bzrlib/bundle/commands.py	2006-06-16 22:29:03 +0000
@@ -15,6 +15,7 @@
 from bzrlib.revision import (common_ancestor, MultipleRevisionSources,
                              NULL_REVISION)
 from bzrlib.revisionspec import RevisionSpec
+from bzrlib.trace import note
 
 
 class cmd_send_changeset(Command):
@@ -86,16 +87,21 @@
         from bzrlib import user_encoding
         from bzrlib.bundle.serializer import write_bundle
 
+        target_branch = Branch.open_containing(u'.')[0]
+
         if base is None:
-            base_branch = None
-        else:
-            base_branch = Branch.open(base)
+            base = target_branch.get_parent()
+            if base is None:
+                raise errors.BzrCommandError("No base branch known or"
+                                             " specified.")
+            else:
+                note('Using saved location: %s' % base)
+        base_branch = Branch.open(base)
 
         # We don't want to lock the same branch across
         # 2 different branches
-        target_branch = Branch.open_containing(u'.')[0]
-        if base_branch is not None and target_branch.base == base_branch.base:
-            base_branch = None
+        if target_branch.base == base_branch.base:
+            base_branch = target_branch 
 
         base_revision = None
         if revision is None:

=== modified file 'bzrlib/bundle/read_bundle.py'
--- bzrlib/bundle/read_bundle.py	2006-06-06 17:56:39 +0000
+++ bzrlib/bundle/read_bundle.py	2006-06-18 19:39:13 +0000
@@ -1,24 +1,40 @@
-#!/usr/bin/env python
-"""\
-Read in a bundle stream, and process it into a BundleReader object.
-"""
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Read in a bundle stream, and process it into a BundleReader object."""
 
 import base64
 from cStringIO import StringIO
 import os
 import pprint
 
+from bzrlib.bundle.common import get_header, header_str
+import bzrlib.errors
 from bzrlib.errors import (TestamentMismatch, BzrError, 
                            MalformedHeader, MalformedPatches, NotABundle)
-from bzrlib.bundle.common import get_header, header_str
 from bzrlib.inventory import (Inventory, InventoryEntry,
                               InventoryDirectory, InventoryFile,
                               InventoryLink)
-from bzrlib.osutils import sha_file, sha_string
+from bzrlib.osutils import sha_file, sha_string, pathjoin
 from bzrlib.revision import Revision, NULL_REVISION
 from bzrlib.testament import StrictTestament
 from bzrlib.trace import mutter, warning
+import bzrlib.transport
 from bzrlib.tree import Tree
+import bzrlib.urlutils
 from bzrlib.xml5 import serializer_v5
 
 
@@ -702,7 +718,7 @@
             if old_dir is None:
                 old_path = None
             else:
-                old_path = os.path.join(old_dir, basename)
+                old_path = pathjoin(old_dir, basename)
         else:
             old_path = new_path
         #If the new path wasn't in renamed, the old one shouldn't be in
@@ -727,7 +743,7 @@
             if new_dir is None:
                 new_path = None
             else:
-                new_path = os.path.join(new_dir, basename)
+                new_path = pathjoin(new_dir, basename)
         else:
             new_path = old_path
         #If the old path wasn't in renamed, the new one shouldn't be in

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-06-20 03:57:11 +0000
+++ bzrlib/bzrdir.py	2006-06-20 05:32:16 +0000
@@ -20,6 +20,8 @@
 directories.
 """
 
+# TODO: remove unittest dependency; put that stuff inside the test suite
+
 from copy import deepcopy
 from cStringIO import StringIO
 import os
@@ -92,11 +94,7 @@
         """
         if not allow_unsupported and not format.is_supported():
             # see open_downlevel to open legacy branches.
-            raise errors.UnsupportedFormatError(
-                    'sorry, format %s not supported' % format,
-                    ['use a different bzr version',
-                     'or remove the .bzr directory'
-                     ' and "bzr init" again'])
+            raise errors.UnsupportedFormatError(format=format)
 
     def clone(self, url, revision_id=None, basis=None, force_new_repo=False):
         """Clone this bzrdir and its contents to url verbatim.
@@ -972,11 +970,13 @@
         """Return the .bzrdir style transport present at URL."""
         try:
             format_string = transport.get(".bzr/branch-format").read()
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+
+        try:
             return klass._formats[format_string]
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
         except KeyError:
-            raise errors.UnknownFormatError(format_string)
+            raise errors.UnknownFormatError(format=format_string)
 
     @classmethod
     def get_default_format(klass):

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-06-20 03:57:11 +0000
+++ bzrlib/commands.py	2006-06-20 05:32:16 +0000
@@ -35,12 +35,13 @@
 import codecs
 
 import bzrlib
+import bzrlib.errors as errors
 from bzrlib.errors import (BzrError,
+                           BzrCommandError,
                            BzrCheckError,
-                           BzrCommandError,
-                           BzrOptionError,
                            NotBranchError)
 from bzrlib.option import Option
+import bzrlib.osutils
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.symbol_versioning import (deprecated_method, zero_eight)
 from bzrlib import trace
@@ -146,7 +147,7 @@
     if cmd_obj:
         return cmd_obj
 
-    raise BzrCommandError("unknown command %r" % cmd_name)
+    raise BzrCommandError('unknown command "%s"' % cmd_name)
 
 
 class Command(object):
@@ -235,17 +236,7 @@
             self.outf = sys.stdout
             return
 
-        output_encoding = getattr(sys.stdout, 'encoding', None)
-        if not output_encoding:
-            input_encoding = getattr(sys.stdin, 'encoding', None)
-            if not input_encoding:
-                output_encoding = bzrlib.user_encoding
-                mutter('encoding stdout as bzrlib.user_encoding %r', output_encoding)
-            else:
-                output_encoding = input_encoding
-                mutter('encoding stdout as sys.stdin encoding %r', output_encoding)
-        else:
-            mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
+        output_encoding = bzrlib.osutils.get_terminal_encoding()
 
         # use 'replace' so that we don't abort if trying to write out
         # in e.g. the default C locale.
@@ -274,8 +265,8 @@
         allowed_names = self.options().keys()
         for oname in opts:
             if oname not in allowed_names:
-                raise BzrCommandError("option '--%s' is not allowed for"
-                                      " command %r" % (oname, self.name()))
+                raise BzrOptionError("option '--%s' is not allowed for"
+                                " command %r" % (oname, self.name()))
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -312,6 +303,17 @@
     def name(self):
         return _unsquish_command_name(self.__class__.__name__)
 
+    def plugin_name(self):
+        """Get the name of the plugin that provides this command.
+
+        :return: The name of the plugin or None if the command is builtin.
+        """
+        mod_parts = self.__module__.split('.')
+        if len(mod_parts) >= 3 and mod_parts[1] == 'plugins':
+            return mod_parts[2]
+        else:
+            return None
+
 
 def parse_spec(spec):
     """
@@ -381,9 +383,7 @@
                     else:
                         optname = a[2:]
                     if optname not in cmd_options:
-                        raise BzrOptionError('unknown long option %r for'
-                                             ' command %s' % 
-                                             (a, command.name()))
+                        raise BzrCommandError('unknown option "%s"' % a)
                 else:
                     shortopt = a[1:]
                     if shortopt in Option.SHORT_OPTIONS:
@@ -398,7 +398,7 @@
                         if shortopt not in Option.SHORT_OPTIONS:
                             # We didn't find the multi-character name, and we
                             # didn't find the single char name
-                            raise BzrError('unknown short option %r' % a)
+                            raise BzrCommandError('unknown option "%s"' % a)
                         optname = Option.SHORT_OPTIONS[shortopt].name
 
                         if a[2:]:
@@ -415,15 +415,12 @@
                                 # This option takes an argument, so pack it
                                 # into the array
                                 optarg = a[2:]
-                
                     if optname not in cmd_options:
-                        raise BzrOptionError('unknown short option %r for'
-                                             ' command %s' % 
-                                             (shortopt, command.name()))
+                        raise BzrCommandError('unknown option "%s"' % shortopt)
                 if optname in opts:
                     # XXX: Do we ever want to support this, e.g. for -r?
                     if proc_aliasarg:
-                        raise BzrError('repeated option %r' % a)
+                        raise BzrCommandError('repeated option %r' % a)
                     elif optname in alias_opts:
                         # Replace what's in the alias with what's in the real
                         # argument
@@ -432,14 +429,14 @@
                         proc_argv.insert(0, a)
                         continue
                     else:
-                        raise BzrError('repeated option %r' % a)
+                        raise BzrCommandError('repeated option %r' % a)
                     
                 option_obj = cmd_options[optname]
                 optargfn = option_obj.type
                 if optargfn:
                     if optarg == None:
                         if not proc_argv:
-                            raise BzrError('option %r needs an argument' % a)
+                            raise BzrCommandError('option %r needs an argument' % a)
                         else:
                             optarg = proc_argv.pop(0)
                     opts[optname] = optargfn(optarg)
@@ -447,7 +444,7 @@
                         alias_opts[optname] = optargfn(optarg)
                 else:
                     if optarg != None:
-                        raise BzrError('option %r takes no argument' % optname)
+                        raise BzrCommandError('option %r takes no argument' % optname)
                     opts[optname] = True
                     if proc_aliasarg:
                         alias_opts[optname] = True
@@ -484,7 +481,7 @@
                 raise BzrCommandError("command %r needs one or more %s"
                         % (cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
-            args[:-1] = []                
+            args[:-1] = []
         else:
             # just a plain arg
             argname = ap
@@ -682,10 +679,7 @@
 def main(argv):
     import bzrlib.ui
     from bzrlib.ui.text import TextUIFactory
-    ## bzrlib.trace.enable_default_logging()
-    trace.log_startup(argv)
     bzrlib.ui.ui_factory = TextUIFactory()
-
     argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
     ret = run_bzr_catch_errors(argv)
     mutter("return code %d", ret)
@@ -702,19 +696,12 @@
     except Exception, e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
-        import errno
-        if (isinstance(e, IOError) 
-            and hasattr(e, 'errno')
-            and e.errno == errno.EPIPE):
-            note('broken pipe')
-            return 3
-        else:
-            trace.log_exception()
-            if os.environ.get('BZR_PDB'):
-                print '**** entering debugger'
-                import pdb
-                pdb.post_mortem(sys.exc_traceback)
-            return 3
+        bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)
+        if os.environ.get('BZR_PDB'):
+            print '**** entering debugger'
+            import pdb
+            pdb.post_mortem(sys.exc_traceback)
+        return 3
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-06-20 03:57:11 +0000
+++ bzrlib/commit.py	2006-06-20 05:32:16 +0000
@@ -75,7 +75,6 @@
 import bzrlib.config
 import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, PointlessCommit,
-                           HistoryMissing,
                            ConflictsInTree,
                            StrictCommitFailed
                            )
@@ -87,10 +86,24 @@
 from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory, ROOT_ID, InventoryEntry
 from bzrlib.symbol_versioning import (deprecated_passed,
+        deprecated_function,
+        zero_seven,
         DEPRECATED_PARAMETER)
 from bzrlib.workingtree import WorkingTree
 
 
+@deprecated_function(zero_seven)
+def commit(*args, **kwargs):
+    """Commit a new revision to a branch.
+
+    Function-style interface for convenience of old callers.
+
+    New code should use the Commit class instead.
+    """
+    ## XXX: Remove this in favor of WorkingTree.commit?
+    Commit().commit(*args, **kwargs)
+
+
 class NullCommitReporter(object):
     """I report on progress of a commit."""
 
@@ -332,6 +345,7 @@
             self._emit_progress_update()
         finally:
             self._cleanup()
+        return self.rev_id
 
     def _check_bound_branch(self):
         """Check to see if the local branch is bound.
@@ -374,14 +388,6 @@
         self.bound_branch = self.branch
         self.master_branch.lock_write()
         self.master_locked = True
-####        
-####        # Check to see if we have any pending merges. If we do
-####        # those need to be pushed into the master branch
-####        pending_merges = self.work_tree.pending_merges()
-####        if pending_merges:
-####            for revision_id in pending_merges:
-####                self.master_branch.repository.fetch(self.bound_branch.repository,
-####                                                    revision_id=revision_id)
 
     def _cleanup(self):
         """Cleanup any open locks, progress bars etc."""
@@ -440,13 +446,8 @@
         """Record the parents of a merge for merge detection."""
         # TODO: Make sure that this list doesn't contain duplicate 
         # entries and the order is preserved when doing this.
-        pending_merges = self.work_tree.pending_merges()
-        self.parents = []
+        self.parents = self.work_tree.get_parent_ids()
         self.parent_invs = []
-        precursor_id = self.branch.last_revision()
-        if precursor_id:
-            self.parents.append(precursor_id)
-        self.parents += pending_merges
         for revision in self.parents:
             if self.branch.repository.has_revision(revision):
                 inventory = self.branch.repository.get_inventory(revision)
@@ -458,9 +459,8 @@
             if not self.branch.repository.has_revision(parent_id):
                 if parent_id == self.branch.last_revision():
                     warning("parent is missing %r", parent_id)
-                    raise HistoryMissing(self.branch, 'revision', parent_id)
-                else:
-                    mutter("commit will ghost revision %r", parent_id)
+                    raise BzrCheckError("branch %s is missing revision {%s}"
+                            % (self.branch, parent_id))
             
     def _remove_deleted(self):
         """Remove deleted files from the working inventories.

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-06-20 03:57:11 +0000
+++ bzrlib/config.py	2006-06-20 05:32:16 +0000
@@ -18,7 +18,7 @@
 """Configuration that affects the behaviour of Bazaar.
 
 Currently this configuration resides in ~/.bazaar/bazaar.conf
-and ~/.bazaar/branches.conf, which is written to by bzr.
+and ~/.bazaar/locations.conf, which is written to by bzr.
 
 In bazaar.conf the following options may be set:
 [DEFAULT]
@@ -29,9 +29,9 @@
 gpg_signing_command=name-of-program
 log_format=name-of-format
 
-in branches.conf, you specify the url of a branch and options for it.
+in locations.conf, you specify the url of a branch and options for it.
 Wildcards may be used - * and ? as normal in shell completion. Options
-set in both bazaar.conf and branches.conf are overridden by the branches.conf
+set in both bazaar.conf and locations.conf are overridden by the locations.conf
 setting.
 [/home/robertc/source]
 recurse=False|True(default)
@@ -49,7 +49,6 @@
 create_signatures - this option controls whether bzr will always create 
                     gpg signatures, never create them, or create them if the
                     branch is configured to require them.
-                    NB: This option is planned, but not implemented yet.
 log_format - This options set the default log format.  Options are long, 
              short, line, or a plugin can register new formats
 
@@ -64,23 +63,29 @@
 
 
 import errno
+from fnmatch import fnmatch
 import os
+import re
 import sys
-from fnmatch import fnmatch
-import re
+from StringIO import StringIO
 
 import bzrlib
 import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, warning
 import bzrlib.util.configobj.configobj as configobj
-from StringIO import StringIO
+
 
 CHECK_IF_POSSIBLE=0
 CHECK_ALWAYS=1
 CHECK_NEVER=2
 
 
+SIGN_WHEN_REQUIRED=0
+SIGN_ALWAYS=1
+SIGN_NEVER=2
+
+
 class ConfigObj(configobj.ConfigObj):
 
     def get_bool(self, section, key):
@@ -106,6 +111,9 @@
     def _get_signature_checking(self):
         """Template method to override signature checking policy."""
 
+    def _get_signing_policy(self):
+        """Template method to override signature creation policy."""
+
     def _get_user_option(self, option_name):
         """Template method to provide a user option."""
         return None
@@ -192,10 +200,24 @@
             return policy
         return CHECK_IF_POSSIBLE
 
+    def signing_policy(self):
+        """What is the current policy for signature checking?."""
+        policy = self._get_signing_policy()
+        if policy is not None:
+            return policy
+        return SIGN_WHEN_REQUIRED
+
     def signature_needed(self):
         """Is a signature needed when committing ?."""
-        policy = self._get_signature_checking()
-        if policy == CHECK_ALWAYS:
+        policy = self._get_signing_policy()
+        if policy is None:
+            policy = self._get_signature_checking()
+            if policy is not None:
+                warning("Please use create_signatures, not check_signatures "
+                        "to set signing policy.")
+            if policy == CHECK_ALWAYS:
+                return True
+        elif policy == SIGN_ALWAYS:
             return True
         return False
 
@@ -232,6 +254,12 @@
         if policy:
             return self._string_to_signature_policy(policy)
 
+    def _get_signing_policy(self):
+        """See Config._get_signing_policy"""
+        policy = self._get_user_option('create_signatures')
+        if policy:
+            return self._string_to_signing_policy(policy)
+
     def _get_user_id(self):
         """Get the user id from the 'email' key in the current section."""
         return self._get_user_option('email')
@@ -272,6 +300,17 @@
         raise errors.BzrError("Invalid signatures policy '%s'"
                               % signature_string)
 
+    def _string_to_signing_policy(self, signature_string):
+        """Convert a string to a signing policy."""
+        if signature_string.lower() == 'when-required':
+            return SIGN_WHEN_REQUIRED
+        if signature_string.lower() == 'never':
+            return SIGN_NEVER
+        if signature_string.lower() == 'always':
+            return SIGN_ALWAYS
+        raise errors.BzrError("Invalid signing policy '%s'"
+                              % signature_string)
+
     def _get_alias(self, value):
         try:
             return self._get_parser().get_value("ALIASES", 
@@ -294,7 +333,12 @@
     """A configuration object that gives the policy for a location."""
 
     def __init__(self, location):
-        super(LocationConfig, self).__init__(branches_config_filename)
+        name_generator = locations_config_filename
+        if (not os.path.exists(name_generator()) and 
+                os.path.exists(branches_config_filename())):
+            warning('Please rename branches.conf to locations.conf')
+            name_generator = branches_config_filename
+        super(LocationConfig, self).__init__(name_generator)
         self._global_config = None
         self.location = location
 
@@ -379,6 +423,13 @@
             return check
         return self._get_global_config()._get_signature_checking()
 
+    def _get_signing_policy(self):
+        """See Config._get_signing_policy."""
+        sign = super(LocationConfig, self)._get_signing_policy()
+        if sign is not None:
+            return sign
+        return self._get_global_config()._get_signing_policy()
+
     def _post_commit(self):
         """See Config.post_commit."""
         hook = self._get_user_option('post_commit')
@@ -389,7 +440,7 @@
     def set_user_option(self, option, value):
         """Save option and its value in the configuration."""
         # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on branches.conf.
+        # file lock on locations.conf.
         conf_dir = os.path.dirname(self._get_filename())
         ensure_config_dir_exists(conf_dir)
         location = self.location
@@ -432,6 +483,10 @@
         """See Config._get_signature_checking."""
         return self._get_location_config()._get_signature_checking()
 
+    def _get_signing_policy(self):
+        """See Config._get_signing_policy."""
+        return self._get_location_config()._get_signing_policy()
+
     def _get_user_option(self, option_name):
         """See Config._get_user_option."""
         return self._get_location_config()._get_user_option(option_name)
@@ -504,6 +559,10 @@
     """Return per-user configuration ini file filename."""
     return pathjoin(config_dir(), 'branches.conf')
 
+def locations_config_filename():
+    """Return per-user configuration ini file filename."""
+    return pathjoin(config_dir(), 'locations.conf')
+
 
 def _auto_user_id():
     """Calculate automatic user identification.

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-06-20 03:57:11 +0000
+++ bzrlib/diff.py	2006-06-20 05:32:16 +0000
@@ -1,30 +1,36 @@
 # Copyright (C) 2004, 2005, 2006 Canonical Ltd.
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import errno
+import os
+import subprocess
+import sys
+import tempfile
 import time
 
 from bzrlib.delta import compare_trees
 from bzrlib.errors import BzrError
 import bzrlib.errors as errors
+import bzrlib.osutils
 from bzrlib.patiencediff import unified_diff
 import bzrlib.patiencediff
 from bzrlib.symbol_versioning import (deprecated_function,
         zero_eight)
 from bzrlib.textfile import check_text_lines
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, warning
 
 
 # TODO: Rather than building a changeset object, we should probably
@@ -82,20 +88,20 @@
 def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
-    import sys
+    if hasattr(to_file, 'fileno'):
+        out_file = to_file
+        have_fileno = True
+    else:
+        out_file = subprocess.PIPE
+        have_fileno = False
     
-    if to_file != sys.stdout:
-        raise NotImplementedError("sorry, can't send external diff other than to stdout yet",
-                                  to_file)
-
     # make sure our own output is properly ordered before the diff
     to_file.flush()
 
-    from tempfile import NamedTemporaryFile
-    import os
-
-    oldtmpf = NamedTemporaryFile()
-    newtmpf = NamedTemporaryFile()
+    oldtmp_fd, old_abspath = tempfile.mkstemp(prefix='bzr-diff-old-')
+    newtmp_fd, new_abspath = tempfile.mkstemp(prefix='bzr-diff-new-')
+    oldtmpf = os.fdopen(oldtmp_fd, 'wb')
+    newtmpf = os.fdopen(newtmp_fd, 'wb')
 
     try:
         # TODO: perhaps a special case for comparing to or from the empty
@@ -108,16 +114,18 @@
         oldtmpf.writelines(oldlines)
         newtmpf.writelines(newlines)
 
-        oldtmpf.flush()
-        newtmpf.flush()
+        oldtmpf.close()
+        newtmpf.close()
 
         if not diff_opts:
             diff_opts = []
         diffcmd = ['diff',
                    '--label', old_filename,
-                   oldtmpf.name,
+                   old_abspath,
                    '--label', new_filename,
-                   newtmpf.name]
+                   new_abspath,
+                   '--binary',
+                  ]
 
         # diff only allows one style to be specified; they don't override.
         # note that some of these take optargs, and the optargs can be
@@ -143,7 +151,19 @@
         if diff_opts:
             diffcmd.extend(diff_opts)
 
-        rc = os.spawnvp(os.P_WAIT, 'diff', diffcmd)
+        try:
+            pipe = subprocess.Popen(diffcmd,
+                                    stdin=subprocess.PIPE,
+                                    stdout=out_file)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise errors.NoDiff(str(e))
+            raise
+        pipe.stdin.close()
+
+        if not have_fileno:
+            bzrlib.osutils.pumpfile(pipe.stdout, to_file)
+        rc = pipe.wait()
         
         if rc != 0 and rc != 1:
             # returns 1 if files differ; that's OK
@@ -156,6 +176,21 @@
     finally:
         oldtmpf.close()                 # and delete
         newtmpf.close()
+        # Clean up. Warn in case the files couldn't be deleted
+        # (in case windows still holds the file open, but not
+        # if the files have already been deleted)
+        try:
+            os.remove(old_abspath)
+        except OSError, e:
+            if e.errno not in (errno.ENOENT,):
+                warning('Failed to delete temporary file: %s %s',
+                        old_abspath, e)
+        try:
+            os.remove(new_abspath)
+        except OSError:
+            if e.errno not in (errno.ENOENT,):
+                warning('Failed to delete temporary file: %s %s',
+                        new_abspath, e)
 
 
 @deprecated_function(zero_eight)
@@ -175,7 +210,6 @@
     supplies any two trees.
     """
     if output is None:
-        import sys
         output = sys.stdout
 
     if from_spec is None:
@@ -222,7 +256,6 @@
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
-    import sys
     output = sys.stdout
     def spec_tree(spec):
         revision_id = spec.in_store(tree.branch).rev_id

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-06-20 04:45:30 +0000
+++ bzrlib/errors.py	2006-06-20 05:32:16 +0000
@@ -16,6 +16,18 @@
 
 """Exceptions for bzr, and reporting of them.
 
+There are 3 different classes of error:
+
+ * KeyboardInterrupt, and OSError with EPIPE - the program terminates 
+   with an appropriate short message
+
+ * User errors, indicating a problem caused by the user such as a bad URL.
+   These are printed in a short form.
+ 
+ * Internal unexpected errors, including most Python builtin errors
+   and some raised from inside bzr.  These are printed with a full 
+   traceback and an invitation to report the bug.
+
 Exceptions are caught at a high level to report errors to the user, and
 might also be caught inside the program.  Therefore it needs to be
 possible to convert them to a meaningful string, and also for them to be
@@ -44,13 +56,14 @@
 Therefore:
 
  * create a new exception class for any class of error that can be
-   usefully distinguished.
-
- * the printable form of an exception is generated by the base class
-   __str__ method
-
-Exception strings should start with a capital letter and not have a final
-fullstop.
+   usefully distinguished.  If no callers are likely to want to catch
+   one but not another, don't worry about them.
+
+ * the __str__ method should generate something useful; BzrError provides
+   a good default implementation
+
+Exception strings should start with a capital letter and should not have a
+final fullstop.
 """
 
 from warnings import warn
@@ -81,6 +94,9 @@
 
 
 class BzrError(StandardError):
+    
+    is_user_error = True
+    
     def __str__(self):
         # XXX: Should we show the exception class in 
         # exceptions that don't provide their own message?  
@@ -119,6 +135,8 @@
 class BzrCheckError(BzrNewError):
     """Internal check failed: %(message)s"""
 
+    is_user_error = False
+
     def __init__(self, message):
         BzrNewError.__init__(self)
         self.message = message
@@ -126,6 +144,9 @@
 
 class InvalidEntryName(BzrNewError):
     """Invalid entry name: %(name)s"""
+
+    is_user_error = False
+
     def __init__(self, name):
         BzrNewError.__init__(self)
         self.name = name
@@ -163,25 +184,31 @@
         self.url = url
 
 
-class BzrCommandError(BzrError):
-    # Error from malformed user command
-    # This is being misused as a generic exception
-    # please subclass. RBC 20051030
+class BzrCommandError(BzrNewError):
+    """Error from user command"""
+
+    is_user_error = True
+
+    # Error from malformed user command; please avoid raising this as a
+    # generic exception not caused by user input.
     #
     # I think it's a waste of effort to differentiate between errors that
     # are not intended to be caught anyway.  UI code need not subclass
     # BzrCommandError, and non-UI code should not throw a subclass of
     # BzrCommandError.  ADHB 20051211
+    def __init__(self, msg):
+        self.msg = msg
+
     def __str__(self):
-        return self.args[0]
+        return self.msg
 
 
 class BzrOptionError(BzrCommandError):
-    """Some missing or otherwise incorrect option was supplied."""
+    """Error in command line options"""
 
     
-class StrictCommitFailed(Exception):
-    """Commit refused because there are unknowns in the tree."""
+class StrictCommitFailed(BzrNewError):
+    """Commit refused because there are unknown files in the tree"""
 
 
 # XXX: Should be unified with TransportError; they seem to represent the
@@ -233,6 +260,9 @@
 
 class PathNotChild(BzrNewError):
     """Path %(path)r is not a child of path %(base)r%(extra)s"""
+
+    is_user_error = False
+
     def __init__(self, path, base, extra=None):
         BzrNewError.__init__(self)
         self.path = path
@@ -280,16 +310,12 @@
         self.path = path
 
 
-class UnsupportedFormatError(BzrError):
-    """Specified path is a bzr branch that we recognize but cannot read."""
-    def __str__(self):
-        return 'unsupported branch format: %s' % self.args[0]
-
-
-class UnknownFormatError(BzrError):
-    """Specified path is a bzr branch whose format we do not recognize."""
-    def __str__(self):
-        return 'unknown branch format: %s' % self.args[0]
+class UnsupportedFormatError(BzrNewError):
+    """Unsupported branch format: %(format)s"""
+
+
+class UnknownFormatError(BzrNewError):
+    """Unknown branch format: %(format)r"""
 
 
 class IncompatibleFormat(BzrNewError):
@@ -376,6 +402,9 @@
 
 class ObjectNotLocked(LockError):
     """%(obj)r is not locked"""
+
+    is_user_error = False
+
     # this can indicate that any particular object is not locked; see also
     # LockNotHeld which means that a particular *lock* object is not held by
     # the caller -- perhaps they should be unified.
@@ -443,12 +472,14 @@
     """Commit refused because there are unknowns in the tree."""
 
 
-class NoSuchRevision(BzrError):
+class NoSuchRevision(BzrNewError):
+    """Branch %(branch)s has no revision %(revision)s"""
+
+    is_user_error = False
+
     def __init__(self, branch, revision):
         self.branch = branch
         self.revision = revision
-        msg = "Branch %s has no revision %s" % (branch, revision)
-        BzrError.__init__(self, msg)
 
 
 class HistoryMissing(BzrError):
@@ -459,26 +490,28 @@
                           % (branch, object_type, object_id))
 
 
-class DivergedBranches(BzrError):
+class DivergedBranches(BzrNewError):
+    "These branches have diverged.  Use the merge command to reconcile them."""
+
+    is_user_error = True
 
     def __init__(self, branch1, branch2):
-        BzrError.__init__(self, "These branches have diverged.  Try merge.")
         self.branch1 = branch1
         self.branch2 = branch2
 
 
-class UnrelatedBranches(BzrCommandError):
-    def __init__(self):
-        msg = "Branches have no common ancestor, and no base revision"\
-            " specified."
-        BzrCommandError.__init__(self, msg)
-
-
-class NoCommonAncestor(BzrError):
+class UnrelatedBranches(BzrNewError):
+    "Branches have no common ancestor, and no merge base revision was specified."
+
+    is_user_error = True
+
+
+class NoCommonAncestor(BzrNewError):
+    "Revisions have no common ancestor: %(revision_a)s %(revision_b)s"
+
     def __init__(self, revision_a, revision_b):
-        msg = "Revisions have no common ancestor: %s %s." \
-            % (revision_a, revision_b) 
-        BzrError.__init__(self, msg)
+        self.revision_a = revision_a
+        self.revision_b = revision_b
 
 
 class NoCommonRoot(BzrError):
@@ -860,6 +893,13 @@
         self.format = format
 
 
+class NoDiff(BzrNewError):
+    """Diff is not installed on this machine: %(msg)s"""
+
+    def __init__(self, msg):
+        super(NoDiff, self).__init__(msg=msg)
+
+
 class NoDiff3(BzrNewError):
     """Diff3 is not installed on this machine."""
 

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-06-20 03:57:11 +0000
+++ bzrlib/fetch.py	2006-06-20 05:32:16 +0000
@@ -126,8 +126,9 @@
         self.from_control = self.from_repository.control_weaves
         self.count_total = 0
         self.file_ids_names = {}
-        pp = ProgressPhase('fetch phase', 4, self.pb)
+        pp = ProgressPhase('Fetch phase', 4, self.pb)
         try:
+            pp.next_phase()
             revs = self._revids_to_fetch()
             # something to do ?
             if revs:

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-05-10 17:48:25 +0000
+++ bzrlib/help.py	2006-06-18 23:19:23 +0000
@@ -85,6 +85,18 @@
     return s
 
 
+def print_command_plugin(cmd_object, outfile, format):
+    """Print the plugin that provides a command object, if any.
+
+    If the cmd_object is provided by a plugin, prints the plugin name to
+    outfile using the provided format string.
+    """
+    plugin_name = cmd_object.plugin_name()
+    if plugin_name is not None:
+        out_str = '(From plugin "%s")' % plugin_name
+        outfile.write(format % out_str)
+
+
 def help_on_command(cmdname, outfile=None):
     from bzrlib.commands import get_cmd_object
 
@@ -107,6 +119,8 @@
 
     print >>outfile
 
+    print_command_plugin(cmd_object, outfile, '%s\n\n')
+
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
@@ -156,6 +170,10 @@
         if cmd_object.hidden:
             continue
         print >>outfile, command_usage(cmd_object)
+
+        plugin_name = cmd_object.plugin_name()
+        print_command_plugin(cmd_object, outfile, '        %s\n')
+
         cmd_help = cmd_object.help()
         if cmd_help:
             firstline = cmd_help.split('\n', 1)[0]

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2006-06-20 03:57:11 +0000
+++ bzrlib/info.py	2006-06-20 05:32:16 +0000
@@ -62,15 +62,15 @@
         branch_path = branch.bzrdir.root_transport.base
         if working_path != branch_path:
             # lightweight checkout
-            print '  light checkout root: %s' % working_path
+            print ' light checkout root: %s' % working_path
             if repository.is_shared():
                 # lightweight checkout of branch in shared repository
-                print '    shared repository: %s' % repository_path
-                print '    repository branch: %s' % (
+                print '   shared repository: %s' % repository_path
+                print '   repository branch: %s' % (
                     _repo_relpath(repository_path, branch_path))
             else:
                 # lightweight checkout of standalone branch
-                print '   checkout of branch: %s' % branch_path
+                print '  checkout of branch: %s' % branch_path
         elif repository.is_shared():
             # branch with tree inside shared repository
             print '    shared repository: %s' % repository_path

=== modified file 'bzrlib/iterablefile.py'
--- bzrlib/iterablefile.py	2006-05-25 17:56:25 +0000
+++ bzrlib/iterablefile.py	2006-06-18 03:45:05 +0000
@@ -15,8 +15,6 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import doctest
-
 class IterableFileBase(object):
     """Create a file-like object from any iterable"""
 
@@ -254,4 +252,5 @@
 
         
 if __name__ == "__main__":
+    import doctest
     doctest.testmod()

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-06-20 03:57:11 +0000
+++ bzrlib/knit.py	2006-06-20 05:32:16 +0000
@@ -2,6 +2,7 @@
 # Written by Martin Pool.
 # Modified by Johan Rydberg <jrydberg@gnu.org>
 # Modified by Robert Collins <robert.collins@canonical.com>
+# Modified by Aaron Bentley <aaron.bentley@utoronto.ca>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,6 +68,7 @@
 from cStringIO import StringIO
 import difflib
 from itertools import izip, chain
+import operator
 import os
 import sys
 
@@ -265,8 +267,8 @@
     stored and retrieved.
     """
 
-    def __init__(self, relpath, transport, file_mode=None, access_mode=None, factory=None,
-                 basis_knit=None, delta=True, create=False):
+    def __init__(self, relpath, transport, file_mode=None, access_mode=None, 
+                 factory=None, basis_knit=None, delta=True, create=False):
         """Construct a knit at location specified by relpath.
         
         :param create: If not True, only open an existing knit.
@@ -510,6 +512,34 @@
             diff_hunks.append((op[1], op[2], op[4]-op[3], new_content._lines[op[3]:op[4]]))
         return diff_hunks
 
+    def _get_component_versions(self, version_id):
+        basis = self.basis_knit
+        needed_versions = []
+        basis_versions = []
+        cursor = version_id
+
+        while 1:
+            picked_knit = self
+            if basis and basis._index.has_version(cursor):
+                picked_knit = basis
+                basis_versions.append(cursor)
+            method = picked_knit._index.get_method(cursor)
+            needed_versions.append((method, cursor))
+            if method == 'fulltext':
+                break
+            cursor = picked_knit.get_parents(cursor)[0]
+        return needed_versions, basis_versions
+
+    def _get_component_positions(self, version_id):
+        needed_versions, basis_versions = \
+            self._get_component_versions(version_id)
+        assert len(basis_versions) == 0
+        positions = []
+        for method, comp_id in needed_versions:
+            data_pos, data_size = self._index.get_position(comp_id)
+            positions.append((method, comp_id, data_pos, data_size))
+        return positions
+
     def _get_components(self, version_id):
         """Return a list of (version_id, method, data) tuples that
         makes up version specified by version_id of the knit.
@@ -533,24 +563,13 @@
         # basis_revisions is a list of versions that needs to be
         # fetched but exists in the basis knit.
 
-        basis = self.basis_knit
-        needed_versions = []
-        basis_versions = []
-        cursor = version_id
-
-        while 1:
-            picked_knit = self
-            if basis and basis._index.has_version(cursor):
-                picked_knit = basis
-                basis_versions.append(cursor)
-            method = picked_knit._index.get_method(cursor)
-            needed_versions.append((method, cursor))
-            if method == 'fulltext':
-                break
-            cursor = picked_knit.get_parents(cursor)[0]
+        needed_versions, basis_versions = \
+            self._get_component_versions(version_id)
 
         components = {}
         if basis_versions:
+            assert False, "I am broken"
+            basis = self.basis_knit
             records = []
             for comp_id in basis_versions:
                 data_pos, data_size = basis._index.get_data_position(comp_id)
@@ -603,7 +622,6 @@
 
         # digest here is the digest from the last applied component.
         if sha_strings(content.text()) != digest:
-            import pdb;pdb.set_trace()
             raise KnitCorrupt(self.filename, 'sha-1 does not match %s' % version_id)
 
         return content
@@ -717,12 +735,71 @@
 
     def _clone_text(self, new_version_id, old_version_id, parents):
         """See VersionedFile.clone_text()."""
-        # FIXME RBC 20060228 make fast by only inserting an index with null delta.
+        # FIXME RBC 20060228 make fast by only inserting an index with null 
+        # delta.
         self.add_lines(new_version_id, parents, self.get_lines(old_version_id))
 
     def get_lines(self, version_id):
         """See VersionedFile.get_lines()."""
-        return self._get_content(version_id).text()
+        return self.get_line_list([version_id])[0]
+
+    def _get_version_components(self, position_map):
+        records = []
+        for version_id, positions in position_map.iteritems():
+            for method, comp_id, position, size in positions:
+                records.append((comp_id, position, size))
+        record_map = self._data.read_records(records)
+
+        component_map = {}
+        for version_id, positions in position_map.iteritems():
+            components = []
+            for method, comp_id, position, size in positions:
+                data, digest = record_map[comp_id]
+                components.append((comp_id, method, data, digest))
+            component_map[version_id] = components
+        return component_map
+
+    def get_text(self, version_id):
+        """See VersionedFile.get_text"""
+        return self.get_texts([version_id])[0]
+
+    def get_texts(self, version_ids):
+        return [''.join(l) for l in self.get_line_list(version_ids)]
+
+    def get_line_list(self, version_ids):
+        """Return the texts of listed versions as a list of strings."""
+        position_map = {}
+        for version_id in version_ids:
+            if not self.has_version(version_id):
+                raise RevisionNotPresent(version_id, self.filename)
+            position_map[version_id] = \
+                self._get_component_positions(version_id)
+
+        version_components = self._get_version_components(position_map).items()
+
+        text_map = {}
+        for version_id, components in version_components:
+            content = None
+            for component_id, method, data, digest in reversed(components):
+                version_idx = self._index.lookup(component_id)
+                if method == 'fulltext':
+                    assert content is None
+                    content = self.factory.parse_fulltext(data, version_idx)
+                elif method == 'line-delta':
+                    delta = self.factory.parse_line_delta(data, version_idx)
+                    content._lines = self._apply_delta(content._lines, delta)
+
+            if 'no-eol' in self._index.get_options(version_id):
+                line = content._lines[-1][1].rstrip('\n')
+                content._lines[-1] = (content._lines[-1][0], line)
+
+            # digest here is the digest from the last applied component.
+            if sha_strings(content.text()) != digest:
+                raise KnitCorrupt(self.filename, 
+                                  'sha-1 does not match %s' % version_id)
+
+            text_map[version_id] = content.text()
+        return [text_map[v] for v in version_ids]
 
     def iter_lines_added_or_present_in_versions(self, version_ids=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
@@ -1373,12 +1450,14 @@
                 needed_records.append((version_id, pos, size))
 
         if len(needed_records):
+            needed_records.sort(key=operator.itemgetter(1))
             # We take it that the transport optimizes the fetching as good
             # as possible (ie, reads continuous ranges.)
             response = self._transport.readv(self._filename,
                 [(pos, size) for version_id, pos, size in needed_records])
 
-            for (record_id, pos, size), (pos, data) in izip(iter(needed_records), response):
+            for (record_id, pos, size), (pos, data) in \
+                izip(iter(needed_records), response):
                 content, digest = self._parse_record(record_id, data)
                 self._records[record_id] = (digest, content)
     

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2006-06-20 03:57:11 +0000
+++ bzrlib/log.py	2006-06-20 05:32:16 +0000
@@ -49,11 +49,9 @@
 all the changes since the previous revision that touched hello.c.
 """
 
-
 # TODO: option to show delta summaries for merged-in revisions
-#
-# TODO: deltas_for_log_reverse seems no longer called; delete it?
 
+from itertools import izip
 import re
 
 from bzrlib.delta import compare_trees
@@ -235,9 +233,22 @@
     for index, rev_id in cut_revs:
         rev_nos[rev_id] = index
 
+    def iter_revisions():
+        revision_ids = [r for s, r, m, e in merge_sorted_revisions]
+        num = 9
+        while revision_ids:
+            revisions = branch.repository.get_revisions(revision_ids[:num])
+            for revision in revisions:
+                yield revision
+            revision_ids  = revision_ids[num:]
+            num = int(num * 1.5)
+            
+        revisions = branch.repository.get_revisions()
+        for revision in revisions:
+            yield revision
     # now we just print all the revisions
-    for sequence, rev_id, merge_depth, end_of_merge in merge_sorted_revisions:
-        rev = branch.repository.get_revision(rev_id)
+    for ((sequence, rev_id, merge_depth, end_of_merge), rev) in \
+        izip(merge_sorted_revisions, iter_revisions()):
 
         if searchRE:
             if not searchRE.search(rev.message):

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2006-06-20 03:57:11 +0000
+++ bzrlib/msgeditor.py	2006-06-20 05:32:16 +0000
@@ -26,7 +26,7 @@
 import bzrlib
 import bzrlib.config as config
 from bzrlib.errors import BzrError
-from bzrlib.trace import warning
+from bzrlib.trace import warning, mutter
 
 
 def _get_editor():
@@ -57,6 +57,7 @@
     for e in _get_editor():
         edargs = e.split(' ')
         try:
+            ## mutter("trying editor: %r", (edargs +[filename]))
             x = call(edargs + [filename])
         except OSError, e:
            # We're searching for an editor, so catch safe errors and continue

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-06-20 03:57:11 +0000
+++ bzrlib/osutils.py	2006-06-20 05:32:16 +0000
@@ -18,8 +18,14 @@
 
 from cStringIO import StringIO
 import errno
+from ntpath import (abspath as _nt_abspath,
+                    join as _nt_join,
+                    normpath as _nt_normpath,
+                    realpath as _nt_realpath,
+                    )
 import os
 from os import listdir
+import posixpath
 import re
 import sha
 import shutil
@@ -33,11 +39,6 @@
 import types
 import tempfile
 import unicodedata
-from ntpath import (abspath as _nt_abspath,
-                    join as _nt_join,
-                    normpath as _nt_normpath,
-                    realpath as _nt_realpath,
-                    )
 
 import bzrlib
 from bzrlib.errors import (BzrError,
@@ -203,24 +204,25 @@
 # string.
 _fs_enc = sys.getfilesystemencoding()
 def _posix_abspath(path):
-    return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
-    # jam 20060426 This is another possibility which mimics 
-    # os.path.abspath, only uses unicode characters instead
-    # if not os.path.isabs(path):
-    #     return os.path.join(os.getcwdu(), path)
-    # return path
+    # jam 20060426 rather than encoding to fsencoding
+    # copy posixpath.abspath, but use os.getcwdu instead
+    if not posixpath.isabs(path):
+        path = posixpath.join(getcwd(), path)
+    return posixpath.normpath(path)
 
 
 def _posix_realpath(path):
-    return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
+    return posixpath.realpath(path.encode(_fs_enc)).decode(_fs_enc)
 
 
 def _win32_abspath(path):
-    return _nt_abspath(path.encode(_fs_enc)).decode(_fs_enc).replace('\\', '/')
+    # Real _nt_abspath doesn't have a problem with a unicode cwd
+    return _nt_abspath(unicode(path)).replace('\\', '/')
 
 
 def _win32_realpath(path):
-    return _nt_realpath(path.encode(_fs_enc)).decode(_fs_enc).replace('\\', '/')
+    # Real _nt_realpath doesn't have a problem with a unicode cwd
+    return _nt_realpath(unicode(path)).replace('\\', '/')
 
 
 def _win32_pathjoin(*args):
@@ -228,7 +230,7 @@
 
 
 def _win32_normpath(path):
-    return _nt_normpath(path).replace('\\', '/')
+    return _nt_normpath(unicode(path)).replace('\\', '/')
 
 
 def _win32_getcwd():
@@ -288,6 +290,33 @@
         return shutil.rmtree(path, ignore_errors, onerror)
 
 
+def get_terminal_encoding():
+    """Find the best encoding for printing to the screen.
+
+    This attempts to check both sys.stdout and sys.stdin to see
+    what encoding they are in, and if that fails it falls back to
+    bzrlib.user_encoding.
+    The problem is that on Windows, locale.getpreferredencoding()
+    is not the same encoding as that used by the console:
+    http://mail.python.org/pipermail/python-list/2003-May/162357.html
+
+    On my standard US Windows XP, the preferred encoding is
+    cp1252, but the console is cp437
+    """
+    output_encoding = getattr(sys.stdout, 'encoding', None)
+    if not output_encoding:
+        input_encoding = getattr(sys.stdin, 'encoding', None)
+        if not input_encoding:
+            output_encoding = bzrlib.user_encoding
+            mutter('encoding stdout as bzrlib.user_encoding %r', output_encoding)
+        else:
+            output_encoding = input_encoding
+            mutter('encoding stdout as sys.stdin encoding %r', output_encoding)
+    else:
+        mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
+    return output_encoding
+
+
 def normalizepath(f):
     if hasattr(os.path, 'realpath'):
         F = realpath
@@ -846,3 +875,18 @@
         for dir in reversed(dirblock):
             if dir[2] == _directory:
                 pending.append(dir)
+
+
+def path_prefix_key(path):
+    """Generate a prefix-order path key for path.
+
+    This can be used to sort paths in the same way that walkdirs does.
+    """
+    return (dirname(path) , path)
+
+
+def compare_paths_prefix_order(path_a, path_b):
+    """Compare path_a and path_b to generate the same order walkdirs uses."""
+    key_a = path_prefix_key(path_a)
+    key_b = path_prefix_key(path_b)
+    return cmp(key_a, key_b)

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2006-02-13 01:28:03 +0000
+++ bzrlib/plugin.py	2006-06-04 21:22:51 +0000
@@ -46,7 +46,7 @@
 
 import bzrlib
 from bzrlib.config import config_dir
-from bzrlib.trace import log_error, mutter, log_exception, warning, \
+from bzrlib.trace import log_error, mutter, warning, \
         log_exception_quietly
 from bzrlib.errors import BzrError
 from bzrlib import plugins

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-06-20 04:45:30 +0000
+++ bzrlib/repository.py	2006-06-20 05:32:16 +0000
@@ -21,7 +21,7 @@
 import time
 from unittest import TestSuite
 
-from bzrlib import bzrdir, check, gpg, errors, xml5, ui, transactions, osutils
+from bzrlib import bzrdir, check, delta, gpg, errors, xml5, ui, transactions, osutils
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.errors import InvalidRevisionId
 from bzrlib.graph import Graph
@@ -39,7 +39,7 @@
         zero_nine, 
         )
 from bzrlib.trace import mutter, note
-from bzrlib.tree import RevisionTree
+from bzrlib.tree import RevisionTree, EmptyTree
 from bzrlib.tsort import topo_sort
 from bzrlib.testament import Testament
 from bzrlib.tree import EmptyTree
@@ -300,8 +300,12 @@
         """
         if not revision_id or not isinstance(revision_id, basestring):
             raise InvalidRevisionId(revision_id=revision_id, branch=self)
-        return self._revision_store.get_revision(revision_id,
-                                                 self.get_transaction())
+        return self._revision_store.get_revisions([revision_id],
+                                                  self.get_transaction())[0]
+    @needs_read_lock
+    def get_revisions(self, revision_ids):
+        return self._revision_store.get_revisions(revision_ids,
+                                                  self.get_transaction())
 
     @needs_read_lock
     def get_revision_xml(self, revision_id):
@@ -327,6 +331,20 @@
         self._check_revision_parents(r, inv)
         return r
 
+    def get_revision_delta(self, revision_id):
+        """Return the delta for one revision.
+
+        The delta is relative to the left-hand predecessor of the
+        revision.
+        """
+        revision = self.get_revision(revision_id)
+        new_tree = self.revision_tree(revision_id)
+        if not revision.parent_ids:
+            old_tree = EmptyTree()
+        else:
+            old_tree = self.revision_tree(revision.parent_ids[0])
+        return delta.compare_trees(old_tree, new_tree)
+
     def _check_revision_parents(self, revision, inventory):
         """Private to Repository and Fetch.
         
@@ -942,7 +960,7 @@
         except errors.NoSuchFile:
             raise errors.NoRepositoryPresent(a_bzrdir)
         except KeyError:
-            raise errors.UnknownFormatError(format_string)
+            raise errors.UnknownFormatError(format=format_string)
 
     def _get_control_store(self, repo_transport, control_files):
         """Return the control store for this repository."""

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2006-06-20 03:57:11 +0000
+++ bzrlib/status.py	2006-06-20 05:32:16 +0000
@@ -30,7 +30,7 @@
 
 
 @deprecated_function(zero_eight)
-def show_status(branch, show_unchanged=False,
+def show_status(branch, show_unchanged=None,
                 specific_files=None,
                 show_ids=False,
                 to_file=None,
@@ -71,7 +71,7 @@
                      specific_files, show_ids, to_file, show_pending, revision)
 
 
-def show_tree_status(wt, show_unchanged=False,
+def show_tree_status(wt, show_unchanged=None,
                      specific_files=None,
                      show_ids=False,
                      to_file=None,
@@ -89,29 +89,24 @@
     If showing the status of a working tree, extra information is included
     about unknown files, conflicts, and pending merges.
 
-    show_unchanged
-        If set, includes unchanged files.
-
-    specific_files
-        If set, a list of filenames whose status should be shown.
-        It is an error to give a filename that is not in the working 
+    :param show_unchanged: Deprecated parameter. If set, includes unchanged 
+        files.
+    :param specific_files: If set, a list of filenames whose status should be
+        shown.  It is an error to give a filename that is not in the working 
         tree, or in the working inventory or in the basis inventory.
-
-    show_ids
-        If set, includes each file's id.
-
-    to_file
-        If set, write to this file (default stdout.)
-
-    show_pending
-        If set, write pending merges.
-
-    revision
-        If None the compare latest revision with working tree
+    :param show_ids: If set, includes each file's id.
+    :param to_file: If set, write to this file (default stdout.)
+    :param show_pending: If set, write pending merges.
+    :param revision: If None the compare latest revision with working tree
+        If not None it must be a RevisionSpec list.
         If one revision show compared it with working tree.
         If two revisions show status between first and second.
     """
-    if to_file == None:
+    if show_unchanged is not None:
+        warn("show_status_trees with show_unchanged has been deprecated "
+             "since bzrlib 0.9", DeprecationWarning, stacklevel=2)
+
+    if to_file is None:
         to_file = sys.stdout
     
     wt.lock_read()

=== modified file 'bzrlib/store/revision/__init__.py'
--- bzrlib/store/revision/__init__.py	2006-03-06 10:17:26 +0000
+++ bzrlib/store/revision/__init__.py	2006-06-12 02:07:37 +0000
@@ -108,6 +108,10 @@
 
     def get_revision(self, revision_id, transaction):
         """Return the Revision object for a named revision."""
+        return self.get_revisions([revision_id], transaction)[0]
+
+    def get_revisions(self, revision_ids, transaction):
+        """Return the Revision objects for a list of named revisions."""
         raise NotImplementedError(self.get_revision)
 
     def get_signature_text(self, revision_id, transaction):

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2006-06-20 04:45:30 +0000
+++ bzrlib/store/revision/knit.py	2006-06-20 05:32:16 +0000
@@ -77,17 +77,28 @@
         rev_file = self.get_revision_file(transaction)
         return rev_file.get_ancestry(rev_file.versions())
 
-    def get_revision(self, revision_id, transaction):
-        """See RevisionStore.get_revision()."""
-        xml = self._get_revision_xml(revision_id, transaction)
+    def get_revisions(self, revision_ids, transaction):
+        """See RevisionStore.get_revisions()."""
+        texts = self._get_serialized_revisions(revision_ids, transaction)
+        revisions = []
         try:
-            r = self._serializer.read_revision_from_string(xml)
+            for text, revision_id in zip(texts, revision_ids):
+                r = self._serializer.read_revision_from_string(text)
+                assert r.revision_id == revision_id
+                revisions.append(r)
         except SyntaxError, e:
             raise errors.BzrError('failed to unpack revision_xml',
                                    [revision_id,
                                    str(e)])
-        assert r.revision_id == revision_id
-        return r
+        return revisions 
+
+    def _get_serialized_revisions(self, revision_ids, transaction):
+        texts = []
+        vf = self.get_revision_file(transaction)
+        try:
+            return vf.get_texts(revision_ids)
+        except (errors.RevisionNotPresent), e:
+            raise errors.NoSuchRevision(self, e.revision_id)
 
     def _get_revision_xml(self, revision_id, transaction):
         try:

=== modified file 'bzrlib/store/revision/text.py'
--- bzrlib/store/revision/text.py	2006-03-08 00:20:04 +0000
+++ bzrlib/store/revision/text.py	2006-06-12 02:07:37 +0000
@@ -85,18 +85,21 @@
                     del parents[parents.index(parent)]
         return topo_sort(result_graph.items())
 
-    def get_revision(self, revision_id, transaction):
-        """See RevisionStore.get_revision()."""
-        xml_file = self._get_revision_xml_file(revision_id)
-        try:
-            r = self._serializer.read_revision(xml_file)
-        except SyntaxError, e:
-            raise errors.BzrError('failed to unpack revision_xml',
+    def get_revisions(self, revision_ids, transaction):
+        """See RevisionStore.get_revisions()."""
+        revisions = []
+        for revision_id in revision_ids:
+            xml_file = self._get_revision_xml_file(revision_id)
+            try:
+                r = self._serializer.read_revision(xml_file)
+            except SyntaxError, e:
+                raise errors.BzrError('failed to unpack revision_xml',
                                    [revision_id,
                                    str(e)])
-            
-        assert r.revision_id == revision_id
-        return r
+            xml_file.close()
+            assert r.revision_id == revision_id
+            revisions.append(r)
+        return revisions 
 
     def _get_revision_xml_file(self, revision_id):
         try:

=== modified file 'bzrlib/tests/EncodingAdapter.py'
--- bzrlib/tests/EncodingAdapter.py	2006-06-07 16:05:27 +0000
+++ bzrlib/tests/EncodingAdapter.py	2006-06-11 03:09:28 +0000
@@ -25,6 +25,15 @@
 # prefix for micro (1/1000000)
 _mu = u'\xb5'
 
+# greek letter omega, not to be confused with
+# the Ohm sign, u'\u2126'. Though they are probably identical
+# cp437 can handle the first, but not the second
+_omega = u'\u03a9'
+
+# smallest error possible, epsilon
+# cp437 handles u03b5, but not u2208 the 'element of' operator
+_epsilon = u'\u03b5'
+
 # Swedish?
 _erik = u'Erik B\xe5gfors'
 
@@ -39,8 +48,8 @@
 
 # Russian, 'Alexander' in russian
 _alexander = u'\u0410\u043b\u0435\u043a\u0441\u0430\u043d\u0434\u0440'
-# No idea if this means anything, but we need another string
-_alex = u'\u0410\u043b\u0435\u043a'
+# The word 'test' in Russian
+_russian_test = u'\u0422\u0435\u0441\u0442'
 
 # Kanji
 # It is a kanji sequence for nihonjin, or Japanese in English.
@@ -100,8 +109,15 @@
                   , 'directory':_something}),
         ('cp1251', 0, {'committer':_alexander
                   , 'message':u'Testing ' + _mu
-                  , 'filename':_alex
-                  , 'directory':_alex + 'dir'}),
+                  , 'filename':_russian_test
+                  , 'directory':_russian_test + 'dir'}),
+# The iso-8859-1 tests run on a default windows cp437 installation
+# and it takes a long time to run an extra permutation of the tests
+# But just in case we want to add this back in:
+#        ('cp437', 0, {'committer':_erik
+#                  , 'message':u'Testing ' + _mu
+#                  , 'filename':'file_' + _omega
+#                  , 'directory':_epsilon + '_dir'}),
     ]
 
     def adapt(self, test):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/__init__.py	2006-06-20 05:32:16 +0000
@@ -338,8 +338,16 @@
                 # If LANG=C we probably have created some bogus paths
                 # which rmtree(unicode) will fail to delete
                 # so make sure we are using rmtree(str) to delete everything
-                osutils.rmtree(test_root.encode(
-                    sys.getfilesystemencoding()))
+                # except on win32, where rmtree(str) will fail
+                # since it doesn't have the property of byte-stream paths
+                # (they are either ascii or mbcs)
+                if sys.platform == 'win32':
+                    # make sure we are using the unicode win32 api
+                    test_root = unicode(test_root)
+                else:
+                    test_root = test_root.encode(
+                        sys.getfilesystemencoding())
+                osutils.rmtree(test_root)
         else:
             if self.pb is not None:
                 self.pb.note("Failed tests working directories are in '%s'\n",
@@ -680,7 +688,6 @@
         self.log('run bzr: %r', argv)
         # FIXME: don't call into logging here
         handler = logging.StreamHandler(stderr)
-        handler.setFormatter(bzrlib.trace.QuietFormatter())
         handler.setLevel(logging.INFO)
         logger = logging.getLogger('')
         logger.addHandler(handler)
@@ -1213,6 +1220,7 @@
                    'bzrlib.tests.test_decorators',
                    'bzrlib.tests.test_diff',
                    'bzrlib.tests.test_doc_generate',
+                   'bzrlib.tests.test_emptytree',
                    'bzrlib.tests.test_errors',
                    'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
@@ -1244,6 +1252,7 @@
                    'bzrlib.tests.test_revision',
                    'bzrlib.tests.test_revisionnamespaces',
                    'bzrlib.tests.test_revprops',
+                   'bzrlib.tests.test_revisiontree',
                    'bzrlib.tests.test_rio',
                    'bzrlib.tests.test_sampler',
                    'bzrlib.tests.test_selftest',
@@ -1273,8 +1282,9 @@
                    'bzrlib.tests.test_xml',
                    ]
     test_transport_implementations = [
-        'bzrlib.tests.test_transport_implementations']
-
+        'bzrlib.tests.test_transport_implementations',
+        'bzrlib.tests.test_read_bundle',
+        ]
     suite = TestUtil.TestSuite()
     loader = TestUtil.TestLoader()
     from bzrlib.transport import TransportTestProviderAdapter

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-06-06 16:16:09 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-06-20 03:30:14 +0000
@@ -46,12 +46,14 @@
                      'bzrlib.tests.blackbox.test_branch',
                      'bzrlib.tests.blackbox.test_break_lock',
                      'bzrlib.tests.blackbox.test_bound_branches',
+                     'bzrlib.tests.blackbox.test_bundle',
                      'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_checkout',
                      'bzrlib.tests.blackbox.test_command_encoding',
                      'bzrlib.tests.blackbox.test_commit',
                      'bzrlib.tests.blackbox.test_conflicts',
                      'bzrlib.tests.blackbox.test_diff',
+                     'bzrlib.tests.blackbox.test_exceptions',
                      'bzrlib.tests.blackbox.test_export',
                      'bzrlib.tests.blackbox.test_find_merge_base',
                      'bzrlib.tests.blackbox.test_help',
@@ -70,6 +72,7 @@
                      'bzrlib.tests.blackbox.test_re_sign',
                      'bzrlib.tests.blackbox.test_revert',
                      'bzrlib.tests.blackbox.test_revno',
+                     'bzrlib.tests.blackbox.test_revision_history',
                      'bzrlib.tests.blackbox.test_revision_info',
                      'bzrlib.tests.blackbox.test_selftest',
                      'bzrlib.tests.blackbox.test_shared_repository',

=== modified file 'bzrlib/tests/blackbox/test_aliases.py'
--- bzrlib/tests/blackbox/test_aliases.py	2006-03-12 20:42:58 +0000
+++ bzrlib/tests/blackbox/test_aliases.py	2006-06-15 05:36:34 +0000
@@ -57,9 +57,9 @@
 
         # If --no-aliases breaks all of bzr, we also get retcode=3
         # So we need to catch the output as well
-        self.assertEquals(bzr_catch_error('--no-aliases', 'c', 'a', 
-                                          retcode=None), 
-                          "bzr: ERROR: unknown command 'c'\n")
+        self.assertEquals(bzr_catch_error('--no-aliases', 'c', 'a',
+                                          retcode=None),
+                          'bzr: ERROR: unknown command "c"\n')
 
         bzr('c', '-r1', '-r2', retcode=3)
         bzr('c1', '-r1', '-r2', retcode=3)

=== modified file 'bzrlib/tests/blackbox/test_ancestry.py'
--- bzrlib/tests/blackbox/test_ancestry.py	2006-05-16 03:02:08 +0000
+++ bzrlib/tests/blackbox/test_ancestry.py	2006-05-31 08:44:29 +0000
@@ -19,12 +19,12 @@
 from bzrlib.builtins import merge
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
+from bzrlib.branch import Branch
 
 
 class TestAncestry(TestCaseWithTransport):
 
-    def test_ancestry(self):
-        """Tests 'ancestry' command"""
+    def _build_branches(self):
         a_wt = self.make_branch_and_tree('A')
         open('A/foo', 'wb').write('1111\n')
         a_wt.add('foo')
@@ -37,13 +37,65 @@
         a_wt.commit('modified A/foo',rev_id='A2')
         merge(['B',-1],['B',1],this_dir='A')
         a_wt.commit('merged B into A',rev_id='A3')
+
+    def _check_ancestry(self, location='', result=None):
+        out = self.capture('ancestry ' + location)
+        if result is None:
+            result = "A1\nB1\nA2\nA3\n"
+        self.assertEqualDiff(out, result)
+
+    def test_ancestry(self):
+        """Tests 'ancestry' command"""
+        self._build_branches()
         os.chdir('A')
-        out, err = self.run_bzr('ancestry')
-        self.assertEquals(err, '')
-        self.assertEqualDiff(out,
-"""A1
-B1
-A2
-A3
-""")
-
+        self._check_ancestry()
+
+    def test_ancestry_with_location(self):
+        """Tests 'ancestry' command with a specified location."""
+        self._build_branches()
+        self._check_ancestry('A')
+
+    def test_ancestry_with_repo_branch(self):
+        """Tests 'ancestry' command with a location that is a
+        repository branch."""
+        self._build_branches()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'A', 'repo/A')
+        self._check_ancestry('repo/A')
+
+    def test_ancestry_with_checkout(self):
+        """Tests 'ancestry' command with a location that is a
+        checkout of a repository branch."""
+        self._build_branches()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'A', 'repo/A')
+        self.run_bzr('checkout', 'repo/A', 'A-checkout')
+        self._check_ancestry('A-checkout')
+
+    def test_ancestry_with_lightweight_checkout(self):
+        """Tests 'ancestry' command with a location that is a
+        lightweight checkout of a repository branch."""
+        self._build_branches()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'A', 'repo/A')
+        self.run_bzr('checkout', '--lightweight', 'repo/A', 'A-checkout')
+        self._check_ancestry('A-checkout')
+
+    def test_ancestry_with_truncated_checkout(self):
+        """Tests 'ancestry' command with a location that is a
+        checkout of a repository branch with a shortened revision history."""
+        self._build_branches()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'A', 'repo/A')
+        self.run_bzr('checkout', '-r', '2', 'repo/A', 'A-checkout')
+        self._check_ancestry('A-checkout', "A1\nA2\n")
+
+    def test_ancestry_with_truncated_lightweight_checkout(self):
+        """Tests 'ancestry' command with a location that is a lightweight
+        checkout of a repository branch with a shortened revision history."""
+        self._build_branches()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'A', 'repo/A')
+        self.run_bzr('checkout', '-r', '2', '--lightweight',
+                'repo/A', 'A-checkout')
+        self._check_ancestry('A-checkout', "A1\nA2\n")

=== added file 'bzrlib/tests/blackbox/test_bundle.py'
--- bzrlib/tests/blackbox/test_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_bundle.py	2006-06-16 22:29:03 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Aaron Bentley
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+from StringIO import StringIO
+
+from bzrlib.bundle.read_bundle import BundleReader
+from bzrlib.bzrdir import BzrDir
+from bzrlib.tests import TestCaseInTempDir
+
+
+class TestBundle(TestCaseInTempDir):
+
+    def test_uses_parent(self):
+        """Parent location is used as a basis by default"""
+        
+        parent_tree = BzrDir.create_standalone_workingtree('parent')
+        parent_tree.commit('initial commit', rev_id='revision1')
+        os.chdir('parent')
+        errmsg = self.run_bzr('bundle', retcode=3)[1]
+        self.assertContainsRe(errmsg, 'No base branch known or specified')
+        branch_tree = parent_tree.bzrdir.sprout('../branch').open_workingtree()
+        branch_tree.commit('next commit', rev_id='revision2')
+        branch_tree.commit('last commit', rev_id='revision3')
+        os.chdir('../branch')
+        br = BundleReader(StringIO(self.run_bzr('bundle')[0]))
+        self.assertEqual(br.info.revisions[0].revision_id, 'revision3')
+        self.assertEqual(len(br.info.revisions), 2)
+        self.assertEqual(br.info.revisions[1].revision_id, 'revision2')

=== added file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2006-06-15 05:49:38 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for display of exceptions."""
+
+import os
+import sys
+
+from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.errors import NotBranchError
+
+class TestExceptionReporting(TestCase):
+
+    def test_report_exception(self):
+        """When an error occurs, display bug report details to stderr"""
+        out, err = self.run_bzr("assert-fail", retcode=3)
+        self.assertContainsRe(err,
+                r'bzr: ERROR: exceptions\.AssertionError: always fails\n')
+        self.assertContainsRe(err, r'please send this report to')
+
+    # TODO: assert-fail doesn't need to always be present; we could just
+    # register (and unregister) it from tests that want to touch it.
+    #
+    # TODO: Some kind of test for the feature of invoking pdb

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-06-13 19:12:10 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-06-16 17:10:52 +0000
@@ -136,6 +136,7 @@
         bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
+        bound_tree = branch3.bzrdir.open_workingtree()
         out, err = self.runbzr('info bound')
         self.assertEqualDiff(
 """Location:
@@ -147,7 +148,7 @@
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
+  working tree: %s
         branch: Branch format 5
     repository: %s
 
@@ -173,6 +174,7 @@
 """ % (branch3.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
+       bound_tree._format.get_format_description(),      
        branch3.repository._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -245,8 +247,8 @@
         out, err = self.runbzr('info lightcheckout')
         self.assertEqualDiff(
 """Location:
-  light checkout root: %s
-   checkout of branch: %s
+ light checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -426,8 +428,8 @@
         out, err = self.runbzr('info lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-  light checkout root: %s
-   checkout of branch: %s
+ light checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -546,81 +548,16 @@
         dir2.create_workingtree()
         tree2 = dir2.open_workingtree()
         branch2 = tree2.branch
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-    shared repository: %s
-    repository branch: branch
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (tree2.bzrdir.root_transport.base,
-       repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
+        self.assertCheckoutStatusOutput('tree/lightcheckout', tree2, shared_repo=repo)
 
         # Create normal checkout
         branch3 = bzrlib.bzrdir.BzrDir.create_branch_convenience('tree/checkout')
         branch3.bind(branch1)
         tree3 = branch3.bzrdir.open_workingtree()
         tree3.update()
-        out, err = self.runbzr('info tree/checkout --verbose')
-        self.assertEqualDiff(
-"""Location:
-       checkout root: %s
-  checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-         0 committers
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (branch3.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-
+        self.assertCheckoutStatusOutput('tree/checkout --verbose', tree3,
+            verbose=True,
+            light_checkout=False, repo_branch=branch1)
         # Update lightweight checkout
         self.build_tree(['tree/lightcheckout/a'])
         tree2.add('a')
@@ -630,9 +567,9 @@
         out, err = self.runbzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-  light checkout root: %s
-    shared repository: %s
-    repository branch: branch
+ light checkout root: %s
+   shared repository: %s
+   repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -759,9 +696,9 @@
         out, err = self.runbzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-  light checkout root: %s
-    shared repository: %s
-    repository branch: branch
+ light checkout root: %s
+   shared repository: %s
+   repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -1150,6 +1087,109 @@
 
         bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
 
+    def assertCheckoutStatusOutput(self, 
+        command_string, lco_tree, shared_repo=None,
+        repo_branch=None,
+        tree_locked=False,
+        branch_locked=False, repo_locked=False,
+        verbose=False,
+        light_checkout=True):
+        """Check the output of info in a light checkout tree.
+
+        This is not quite a mirror of the info code: rather than using the
+        tree being examined to predict output, it uses a bunch of flags which
+        allow us, the test writers, to document what *should* be present in
+        the output. Removing this separation would remove the value of the
+        tests.
+        
+        :param path: the path to the light checkout.
+        :param lco_tree: the tree object for the light checkout.
+        :param shared_repo: A shared repository is in use, expect that in
+            the output.
+        :param repo_branch: A branch in a shared repository for non light
+            checkouts.
+        :param tree_locked: If true, expect the tree to be locked.
+        :param branch_locked: If true, expect the branch to be locked.
+        :param repo_locked: If true, expect the repository to be locked.
+        :param verbose: If true, expect verbose output
+        """
+        out, err = self.runbzr('info %s' % command_string)
+        if repo_locked or branch_locked or tree_locked:
+            def locked_message(a_bool):
+                if a_bool:
+                    return 'locked'
+                else:
+                    return 'unlocked'
+            expected_lock_output = (
+                "\n"
+                "Lock status:\n"
+                "  working tree: %s\n"
+                "        branch: %s\n"
+                "    repository: %s\n" % (
+                    locked_message(tree_locked),
+                    locked_message(branch_locked),
+                    locked_message(repo_locked)))
+        else:
+            expected_lock_output = ''
+        if light_checkout:
+            tree_data = (" light checkout root: %s" %
+                lco_tree.bzrdir.root_transport.base)
+        else:
+            tree_data = ("       checkout root: %s" %
+                lco_tree.bzrdir.root_transport.base)
+        if shared_repo is not None:
+            branch_data = (
+                "   shared repository: %s\n"
+                "   repository branch: branch\n" %
+                shared_repo.bzrdir.root_transport.base)
+        elif repo_branch is not None:
+            branch_data = (
+                "  checkout of branch: %s\n" % 
+                repo_branch.bzrdir.root_transport.base)
+        else:
+            branch_data = ("  checkout of branch: %s\n" % 
+                lco_tree.branch.bzrdir.root_transport.base)
+        
+        if verbose:
+            verbose_info = '         0 committers\n'
+        else:
+            verbose_info = ''
+            
+        self.assertEqualDiff(
+"""Location:
+%s
+%s
+Format:
+       control: Meta directory format 1
+  working tree: %s
+        branch: Branch format 5
+    repository: %s
+%s
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+%s
+Revision store:
+         0 revisions
+         0 KiB
+""" %  (tree_data,
+        branch_data,
+        lco_tree._format.get_format_description(),
+        lco_tree.branch.repository._format.get_format_description(),
+        expected_lock_output,
+        verbose_info,
+        ), out)
+        self.assertEqual('', err)
+
     def test_info_locking(self):
         transport = self.get_transport()
         # Create shared repository with a branch
@@ -1176,336 +1216,78 @@
         # W B R
 
         # U U U
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
+        self.assertCheckoutStatusOutput('tree/lightcheckout', lco_tree)
         # U U L
         lco_tree.branch.repository.lock_write()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: unlocked
-        branch: unlocked
-    repository: locked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.repository.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            repo_locked=True)
+        finally:
+            lco_tree.branch.repository.unlock()
         # U L L
         lco_tree.branch.lock_write()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: unlocked
-        branch: locked
-    repository: locked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            branch_locked=True,
+            repo_locked=True)
+        finally:
+            lco_tree.branch.unlock()
         # L L L
         lco_tree.lock_write()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: locked
-        branch: locked
-    repository: locked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            tree_locked=True,
+            branch_locked=True,
+            repo_locked=True)
+        finally:
+            lco_tree.unlock()
         # L L U
         lco_tree.lock_write()
         lco_tree.branch.repository.unlock()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: locked
-        branch: locked
-    repository: unlocked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.repository.lock_write()
-        lco_tree.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            tree_locked=True,
+            branch_locked=True)
+        finally:
+            lco_tree.branch.repository.lock_write()
+            lco_tree.unlock()
         # L U U
         lco_tree.lock_write()
         lco_tree.branch.unlock()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: locked
-        branch: unlocked
-    repository: unlocked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.lock_write()
-        lco_tree.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            tree_locked=True)
+        finally:
+            lco_tree.branch.lock_write()
+            lco_tree.unlock()
         # L U L
         lco_tree.lock_write()
         lco_tree.branch.unlock()
         lco_tree.branch.repository.lock_write()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: locked
-        branch: unlocked
-    repository: locked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.repository.unlock()
-        lco_tree.branch.lock_write()
-        lco_tree.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            tree_locked=True,
+            repo_locked=True)
+        finally:
+            lco_tree.branch.repository.unlock()
+            lco_tree.branch.lock_write()
+            lco_tree.unlock()
         # U L U
         lco_tree.branch.lock_write()
         lco_tree.branch.repository.unlock()
-        out, err = self.runbzr('info tree/lightcheckout')
-        self.assertEqualDiff(
-"""Location:
-  light checkout root: %s
-   checkout of branch: %s
-
-Format:
-       control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
-    repository: %s
-
-Lock status:
-  working tree: unlocked
-        branch: locked
-    repository: unlocked
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Revision store:
-         0 revisions
-         0 KiB
-""" % (lco_tree.bzrdir.root_transport.base,
-       lco_tree.branch.bzrdir.root_transport.base,
-       lco_tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        lco_tree.branch.repository.lock_write()
-        lco_tree.branch.unlock()
+        try:
+            self.assertCheckoutStatusOutput('tree/lightcheckout',
+            lco_tree,
+            branch_locked=True)
+        finally:
+            lco_tree.branch.repository.lock_write()
+            lco_tree.branch.unlock()
 
     def test_info_locking_oslocks(self):
         if sys.platform == "win32":

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2006-05-05 01:29:34 +0000
+++ bzrlib/tests/blackbox/test_init.py	2006-06-15 05:36:34 +0000
@@ -70,9 +70,9 @@
         
         out, err = self.run_bzr('init', 'subdir2/nothere', retcode=3)
         self.assertEqual('', out)
-        self.failUnless(err.startswith(
-            'bzr: ERROR: exceptions.OSError: '
-            '[Errno 2] No such file or directory: '))
+        self.assertContainsRe(err,
+            r'^bzr: ERROR: .*'
+            '\[Errno 2\] No such file or directory: ')
         
         os.mkdir('subdir2')
         out, err = self.run_bzr('init', 'subdir2')

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-06-07 16:05:27 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-06-11 02:57:43 +0000
@@ -21,6 +21,7 @@
 import os
 
 import bzrlib
+import bzrlib.osutils as osutils
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.trace import mutter, note
 import bzrlib.urlutils as urlutils
@@ -52,6 +53,7 @@
         bzr = self.run_bzr
 
         fs_enc = sys.getfilesystemencoding()
+        terminal_enc = osutils.get_terminal_encoding()
         fname = self.info['filename']
         dir_name = self.info['directory']
         for thing in [fname, dir_name]:
@@ -59,8 +61,16 @@
                 thing.encode(fs_enc)
             except UnicodeEncodeError:
                 raise TestSkipped(('Unable to represent path %r'
-                                   ' in filesystem encoding %s')
+                                   ' in filesystem encoding "%s"')
                                     % (thing, fs_enc))
+            try:
+                thing.encode(terminal_enc)
+            except UnicodeEncodeError:
+                raise TestSkipped(('Unable to represent path %r'
+                                   ' in terminal encoding "%s"'
+                                   ' (even though it is valid in'
+                                   ' filesystem encoding "%s")')
+                                   % (thing, terminal_enc, fs_enc))
 
         bzr('init')
         open('a', 'wb').write('foo\n')
@@ -208,7 +218,7 @@
         open('a', 'ab').write('more text\n')
         bzr('commit', '-m', 'mod a')
 
-        pwd = os.getcwdu()
+        pwd = osutils.getcwd()
 
         os.chdir(u'../' + dirname2)
         txt = bzr('pull')

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2006-04-27 02:21:56 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2006-06-20 03:30:14 +0000
@@ -247,7 +247,7 @@
         tree_b.commit('commit d')
         out = self.runbzr('pull ../branch_a', retcode=3)
         self.assertEquals(out,
-                ('','bzr: ERROR: These branches have diverged.  Try merge.\n'))
+                ('','bzr: ERROR: These branches have diverged.  Use the merge command to reconcile them.\n'))
         self.assertEquals(branch_b.get_parent(), parent)
         # test implicit --remember after resolving previous failure
         uncommit(branch=branch_b, tree=tree_b)
@@ -258,3 +258,48 @@
         self.runbzr('pull ../branch_c --remember')
         self.assertEquals(branch_b.get_parent(),
                           branch_c.bzrdir.root_transport.base)
+
+    def test_pull_bundle(self):
+        from bzrlib.testament import Testament
+        # Build up 2 trees and prepare for a pull
+        tree_a = self.make_branch_and_tree('branch_a')
+        f = open('branch_a/a', 'wb')
+        f.write('hello')
+        f.close()
+        tree_a.add('a')
+        tree_a.commit('message')
+
+        tree_b = tree_a.bzrdir.sprout('branch_b').open_workingtree()
+
+        # Make a change to 'a' that 'b' can pull
+        f = open('branch_a/a', 'wb')
+        f.write('hey there')
+        f.close()
+        tree_a.commit('message')
+
+        # Create the bundle for 'b' to pull
+        os.chdir('branch_a')
+        bundle_file = open('../bundle', 'wb')
+        bundle_file.write(self.run_bzr('bundle', '../branch_b')[0])
+        bundle_file.close()
+
+        os.chdir('../branch_b')
+        output = self.run_bzr('pull', '../bundle')
+        self.assertEqual('', output[0])
+        self.assertEqual('All changes applied successfully.\n'
+                         '1 revision(s) pulled.\n', output[1])
+
+        self.assertEqualDiff(tree_a.branch.revision_history(),
+                             tree_b.branch.revision_history())
+
+        testament_a = Testament.from_revision(tree_a.branch.repository, 
+                                              tree_a.last_revision())
+        testament_b = Testament.from_revision(tree_b.branch.repository,
+                                              tree_b.last_revision())
+        self.assertEqualDiff(testament_a.as_text(),
+                             testament_b.as_text())
+
+        # it is legal to attempt to pull an already-merged bundle
+        output = self.run_bzr('pull', '../bundle')
+        self.assertEqual('', output[0])
+        self.assertEqual('0 revision(s) pulled.\n', output[1])

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2006-05-21 13:58:58 +0000
+++ bzrlib/tests/blackbox/test_push.py	2006-06-17 00:49:34 +0000
@@ -27,6 +27,7 @@
 from bzrlib.repository import RepositoryFormatKnit1
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.uncommit import uncommit
+from bzrlib.urlutils import local_path_from_url
 from bzrlib.workingtree import WorkingTree
 
 
@@ -52,28 +53,43 @@
         tree_b.commit('commit c')
         # initial push location must be empty
         self.assertEqual(None, branch_b.get_push_location())
+
         # test push for failure without push location set
         os.chdir('branch_a')
         out = self.runbzr('push', retcode=3)
         self.assertEquals(out,
                 ('','bzr: ERROR: No push location known or specified.\n'))
+
+        # test not remembered if cannot actually push
+        self.run_bzr('push', '../path/which/doesnt/exist', retcode=3)
+        out = self.run_bzr('push', retcode=3)
+        self.assertEquals(
+                ('', 'bzr: ERROR: No push location known or specified.\n'),
+                out)
+
         # test implicit --remember when no push location set, push fails
-        out = self.runbzr('push ../branch_b', retcode=3)
+        out = self.run_bzr('push', '../branch_b', retcode=3)
         self.assertEquals(out,
                 ('','bzr: ERROR: These branches have diverged.  '
                     'Try a merge then push with overwrite.\n'))
         self.assertEquals(abspath(branch_a.get_push_location()),
                           abspath(branch_b.bzrdir.root_transport.base))
+
         # test implicit --remember after resolving previous failure
         uncommit(branch=branch_b, tree=tree_b)
         transport.delete('branch_b/c')
-        self.runbzr('push')
-        self.assertEquals(abspath(branch_a.get_push_location()),
-                          abspath(branch_b.bzrdir.root_transport.base))
+        out = self.run_bzr('push')
+        path = branch_a.get_push_location()
+        self.assertEquals(('Using saved location: %s\n' 
+                           % (local_path_from_url(path),)
+                          , 'All changes applied successfully.\n'
+                            '1 revision(s) pushed.\n'), out)
+        self.assertEqual(path,
+                         branch_b.bzrdir.root_transport.base)
         # test explicit --remember
-        self.runbzr('push ../branch_c --remember')
-        self.assertEquals(abspath(branch_a.get_push_location()),
-                          abspath(branch_c.bzrdir.root_transport.base))
+        self.run_bzr('push', '../branch_c', '--remember')
+        self.assertEquals(branch_a.get_push_location(),
+                          branch_c.bzrdir.root_transport.base)
     
     def test_push_without_tree(self):
         # bzr push from a branch that does not have a checkout should work.

=== added file 'bzrlib/tests/blackbox/test_revision_history.py'
--- bzrlib/tests/blackbox/test_revision_history.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_revision_history.py	2006-05-30 15:03:34 +0000
@@ -0,0 +1,75 @@
+# Copyright (C) 2006, Michael Ellerman
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+
+
+class TestRevisionHistory(TestCaseWithTransport):
+
+    def _build_branch(self):
+        # setup a standalone branch with three commits
+        tree = self.make_branch_and_tree('test')
+        open('test/foo', 'wb').write('1111\n')
+        tree.add('foo')
+        tree.commit('added foo',rev_id='revision_1')
+        open('test/foo', 'wb').write('2222\n')
+        tree.commit('updated foo',rev_id='revision_2')
+        open('test/foo', 'wb').write('3333\n')
+        tree.commit('updated foo again',rev_id='revision_3')
+        return tree
+
+    def _check_revision_history(self, location=''):
+        rh = self.capture('revision-history ' + location)
+        self.assertEqual(rh, 'revision_1\nrevision_2\nrevision_3\n')
+
+    def test_revision_history(self):
+        """Tests 'revision_history' command"""
+        self._build_branch()
+        os.chdir('test')
+        self._check_revision_history()
+
+    def test_revision_history_with_location(self):
+        """Tests 'revision_history' command with a specified location."""
+        self._build_branch()
+        self._check_revision_history('test')
+
+    def test_revision_history_with_repo_branch(self):
+        """Tests 'revision_history' command with a location that is a
+        repository branch."""
+        self._build_branch()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'test', 'repo/test')
+        self._check_revision_history('repo/test')
+
+    def test_revision_history_with_checkout(self):
+        """Tests 'revision_history' command with a location that is a
+        checkout of a repository branch."""
+        self._build_branch()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'test', 'repo/test')
+        self.run_bzr('checkout', 'repo/test', 'test-checkout')
+        self._check_revision_history('test-checkout')
+
+    def test_revision_history_with_lightweight_checkout(self):
+        """Tests 'revision_history' command with a location that is a
+        lightweight checkout of a repository branch."""
+        self._build_branch()
+        self.run_bzr('init-repo', 'repo')
+        self.run_bzr('branch', 'test', 'repo/test')
+        self.run_bzr('checkout', '--lightweight', 'repo/test', 'test-checkout')
+        self._check_revision_history('test-checkout')

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-06-06 22:59:58 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-06-18 03:39:50 +0000
@@ -696,12 +696,6 @@
         out = capture("status")
         self.assertEquals(out, 'unknown:\n  test.txt\n')
 
-        out = capture("status --all")
-        self.assertEquals(out, "unknown:\n  test.txt\n")
-
-        out = capture("status test.txt --all")
-        self.assertEquals(out, "unknown:\n  test.txt\n")
-
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
@@ -715,7 +709,7 @@
         os.unlink('test2.txt')
 
         progress("command aliases")
-        out = capture("st --all")
+        out = capture("st")
         self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
         out = capture("stat")
@@ -737,7 +731,6 @@
 
         runbzr("add test.txt")
         self.assertEquals(capture("unknowns"), '')
-        self.assertEquals(capture("status --all"), ("added:\n" "  test.txt\n"))
 
         progress("rename newly-added file")
         runbzr("rename test.txt hello.txt")

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-06-20 05:32:16 +0000
@@ -21,6 +21,7 @@
 
 from bzrlib import branch, bzrdir, errors, gpg, transactions, repository
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.delta import TreeDelta
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
@@ -96,6 +97,22 @@
         tree = b2.repository.revision_tree('revision-1')
         self.assertEqual(tree.get_file_text('foo-id'), 'hello')
 
+    def test_get_revision_delta(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        self.build_tree(['a/vla'])
+        tree_a.add('vla', 'file2')
+        tree_a.commit('rev2', rev_id='rev2')
+
+        delta = tree_a.branch.get_revision_delta(1)
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('foo', 'file1', 'file')], delta.added)
+        delta = tree_a.branch.get_revision_delta(2)
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('vla', 'file2', 'file')], delta.added)
+
     def get_unbalanced_tree_pair(self):
         """Return two branches, a and b, with one file in a."""
         get_transport(self.get_url()).mkdir('a')
@@ -467,20 +484,20 @@
         self.assertEqual(None, self.get_branch().get_push_location())
 
     def test_get_push_location_exact(self):
-        from bzrlib.config import (branches_config_filename,
+        from bzrlib.config import (locations_config_filename,
                                    ensure_config_dir_exists)
         ensure_config_dir_exists()
-        fn = branches_config_filename()
+        fn = locations_config_filename()
         print >> open(fn, 'wt'), ("[%s]\n"
                                   "push_location=foo" %
                                   self.get_branch().base[:-1])
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
-        from bzrlib.config import (branches_config_filename,
+        from bzrlib.config import (locations_config_filename,
                                    ensure_config_dir_exists)
         ensure_config_dir_exists()
-        fn = branches_config_filename()
+        fn = locations_config_filename()
         self.get_branch().set_push_location('foo')
         self.assertFileEqual("[%s]\n"
                              "push_location = foo" % self.get_branch().base[:-1],

=== modified file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	2006-06-06 22:37:30 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2006-06-15 19:13:50 +0000
@@ -15,7 +15,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+import cStringIO
 import os
+import sys
 
 from bzrlib.branch import Branch
 import bzrlib.errors
@@ -32,34 +34,43 @@
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
         b = self.make_branch('.')
-        self.assertEquals(b.get_parent(), None)
+        self.assertEqual(None, b.get_parent())
         
     def test_set_get_parent(self):
         """Set, re-get and reset the parent"""
         b = self.make_branch('.')
         url = 'http://bazaar-vcs.org/bzr/bzr.dev'
         b.set_parent(url)
-        self.assertEquals(b.get_parent(), url)
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), url)
+        self.assertEqual(url, b.get_parent())
+        self.assertEqual(url, b.control_files.get('parent').read().strip('\n'))
 
         b.set_parent(None)
-        self.assertEquals(b.get_parent(), None)
+        self.assertEqual(None, b.get_parent())
 
         b.set_parent('../other_branch')
         cwd = getcwd()
 
-        self.assertEquals(b.get_parent(), local_path_to_url('../other_branch'))
+        self.assertEqual(local_path_to_url('../other_branch'), b.get_parent())
         path = local_path_to_url('../yanb')
         b.set_parent(path)
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), 
-            '../yanb')
-        self.assertEqual(b.get_parent(), path)
+        self.assertEqual('../yanb',
+            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual(path, b.get_parent())
 
 
         self.assertRaises(bzrlib.errors.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(escape(u'\xb5'))
-        self.assertEqual(b.control_files.get('parent').read().strip('\n'), 
-            '%C2%B5')
-
-        self.assertEqual(b.get_parent(), b.base + '%C2%B5')
+        self.assertEqual('%C2%B5', 
+            b.control_files.get('parent').read().strip('\n'))
+
+        self.assertEqual(b.base + '%C2%B5', b.get_parent())
+
+        # Handle the case for older style absolute local paths
+        if sys.platform == 'win32':
+            # TODO: jam 20060515 Do we want to special case Windows local
+            #       paths as well? Nobody has complained about it.
+            pass
+        else:
+            b.control_files.put('parent', cStringIO.StringIO('/local/abs/path'))
+            self.assertEqual('file:///local/abs/path', b.get_parent())
 

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-06-20 05:32:16 +0000
@@ -21,16 +21,15 @@
 import sys
 
 import bzrlib
-import bzrlib.bzrdir as bzrdir
+from bzrlib import bzrdir, errors, repository
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-import bzrlib.errors as errors
+from bzrlib.delta import TreeDelta
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
                            UninitializableFormat,
                            NotBranchError,
                            )
-import bzrlib.repository as repository
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
@@ -166,6 +165,22 @@
                    revision_id=None,
                    pb=bzrlib.progress.DummyProgress())
 
+    def test_get_revision_delta(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        self.build_tree(['a/vla'])
+        tree_a.add('vla', 'file2')
+        tree_a.commit('rev2', rev_id='rev2')
+
+        delta = tree_a.branch.repository.get_revision_delta('rev1')
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('foo', 'file1', 'file')], delta.added)
+        delta = tree_a.branch.repository.get_revision_delta('rev2')
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('vla', 'file2', 'file')], delta.added)
+
     def test_clone_bzrdir_repository_revision(self):
         # make a repository with some revisions,
         # and clone it, this should not have unreferenced revisions.
@@ -279,6 +294,23 @@
         result.report_results(verbose=True)
         result.report_results(verbose=False)
 
+    def test_get_revisions(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('initial empty commit', rev_id='a-rev',
+                    allow_pointless=True)
+        tree.commit('second empty commit', rev_id='b-rev',
+                    allow_pointless=True)
+        tree.commit('third empty commit', rev_id='c-rev',
+                    allow_pointless=True)
+        repo = tree.branch.repository
+        revision_ids = ['a-rev', 'b-rev', 'c-rev']
+        revisions = repo.get_revisions(revision_ids)
+        assert len(revisions) == 3, repr(revisions)
+        zipped = zip(revisions, revision_ids)
+        self.assertEqual(len(zipped), 3)
+        for revision, revision_id in zipped:
+            self.assertEqual(revision.revision_id, revision_id)
+            self.assertEqual(revision, repo.get_revision(revision_id))
 
 class TestCaseWithComplexRepository(TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2006-06-07 11:06:23 +0000
+++ bzrlib/tests/stub_sftp.py	2006-06-15 21:16:20 +0000
@@ -143,7 +143,7 @@
             fstr = 'rb'
         try:
             f = os.fdopen(fd, fstr)
-        except OSError, e:
+        except (IOError, OSError), e:
             return SFTPServer.convert_errno(e.errno)
         fobj = StubSFTPHandle()
         fobj.filename = path

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2006-05-17 08:41:50 +0000
+++ bzrlib/tests/test_config.py	2006-06-16 15:59:24 +0000
@@ -41,15 +41,18 @@
 
 
 sample_always_signatures = ("[DEFAULT]\n"
-                            "check_signatures=require\n")
+                            "check_signatures=ignore\n"
+                            "create_signatures=always")
 
 
 sample_ignore_signatures = ("[DEFAULT]\n"
-                            "check_signatures=ignore\n")
+                            "check_signatures=require\n"
+                            "create_signatures=never")
 
 
 sample_maybe_signatures = ("[DEFAULT]\n"
-                            "check_signatures=check-available\n")
+                            "check_signatures=ignore\n"
+                            "create_signatures=when-required")
 
 
 sample_branches_text = ("[http://www.example.com]\n"
@@ -172,8 +175,11 @@
 
     def test_signatures_default(self):
         my_config = config.Config()
+        self.assertFalse(my_config.signature_needed())
         self.assertEqual(config.CHECK_IF_POSSIBLE,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_WHEN_REQUIRED,
+                         my_config.signing_policy())
 
     def test_signatures_template_method(self):
         my_config = InstrumentedConfig()
@@ -248,6 +254,14 @@
             self.assertEqual(config.branches_config_filename(),
                              '/home/bogus/.bazaar/branches.conf')
 
+    def test_locations_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.locations_config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/locations.conf')
+        else:
+            self.assertEqual(config.locations_config_filename(),
+                             '/home/bogus/.bazaar/locations.conf')
+
 class TestIniConfig(TestCase):
 
     def test_contructs(self):
@@ -326,24 +340,30 @@
         config_file = StringIO(sample_always_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_ALWAYS,
+        self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_ALWAYS,
+                         my_config.signing_policy())
         self.assertEqual(True, my_config.signature_needed())
 
     def test_signatures_if_possible(self):
         config_file = StringIO(sample_maybe_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_IF_POSSIBLE,
+        self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_WHEN_REQUIRED,
+                         my_config.signing_policy())
         self.assertEqual(False, my_config.signature_needed())
 
     def test_signatures_ignore(self):
         config_file = StringIO(sample_ignore_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_NEVER,
+        self.assertEqual(config.CHECK_ALWAYS,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_NEVER,
+                         my_config.signing_policy())
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_sample_config(self):
@@ -410,15 +430,26 @@
         # replace the class that is constructured, to check its parameters
         oldparserclass = config.ConfigObj
         config.ConfigObj = InstrumentedConfigObj
-        my_config = config.LocationConfig('http://www.example.com')
         try:
+            my_config = config.LocationConfig('http://www.example.com')
             parser = my_config._get_parser()
         finally:
             config.ConfigObj = oldparserclass
         self.failUnless(isinstance(parser, InstrumentedConfigObj))
         self.assertEqual(parser._calls,
-                         [('__init__', config.branches_config_filename(),
+                         [('__init__', config.locations_config_filename(),
                            'utf-8')])
+        os.mkdir(config.config_dir())
+        f = file(config.branches_config_filename(), 'wb')
+        f.write('')
+        f.close()
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        try:
+            my_config = config.LocationConfig('http://www.example.com')
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
 
     def test_get_global_config(self):
         my_config = config.LocationConfig('http://example.com')
@@ -495,8 +526,10 @@
     def test_signatures_not_set(self):
         self.get_location_config('http://www.example.com',
                                  global_config=sample_ignore_signatures)
-        self.assertEqual(config.CHECK_NEVER,
+        self.assertEqual(config.CHECK_ALWAYS,
                          self.my_config.signature_checking())
+        self.assertEqual(config.SIGN_NEVER,
+                         self.my_config.signing_policy())
 
     def test_signatures_never(self):
         self.get_location_config('/a/c')
@@ -609,7 +642,9 @@
         config_file = StringIO(sample_always_signatures)
         (my_config._get_location_config().
             _get_global_config()._get_parser(config_file))
-        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(config.SIGN_ALWAYS, my_config.signing_policy())
+        self.assertTrue(my_config.signature_needed())
 
     def test_gpg_signing_command(self):
         branch = FakeBranch()

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/test_diff.py	2006-06-20 05:32:16 +0000
@@ -16,11 +16,14 @@
 
 import os
 from cStringIO import StringIO
+import errno
+from tempfile import TemporaryFile
 
-from bzrlib.diff import internal_diff, show_diff_trees
-from bzrlib.errors import BinaryFile
+from bzrlib.diff import internal_diff, external_diff, show_diff_trees
+from bzrlib.errors import BinaryFile, NoDiff
 import bzrlib.patiencediff
-from bzrlib.tests import TestCase, TestCaseWithTransport, TestCaseInTempDir
+from bzrlib.tests import (TestCase, TestCaseWithTransport,
+                          TestCaseInTempDir, TestSkipped)
 
 
 def udiff_lines(old, new, allow_binary=False):
@@ -30,6 +33,22 @@
     return output.readlines()
 
 
+def external_udiff_lines(old, new, use_stringio=False):
+    if use_stringio:
+        # StringIO has no fileno, so it tests a different codepath
+        output = StringIO()
+    else:
+        output = TemporaryFile()
+    try:
+        external_diff('old', old, 'new', new, output, diff_opts=['-u'])
+    except NoDiff:
+        raise TestSkipped('external "diff" not present to test')
+    output.seek(0, 0)
+    lines = output.readlines()
+    output.close()
+    return lines
+
+
 class TestDiff(TestCase):
 
     def test_add_nl(self):
@@ -77,6 +96,18 @@
         udiff_lines([1023 * 'a' + '\x00'], [], allow_binary=True)
         udiff_lines([], [1023 * 'a' + '\x00'], allow_binary=True)
 
+    def test_external_diff(self):
+        lines = external_udiff_lines(['boo\n'], ['goo\n'])
+        self.check_patch(lines)
+
+    def test_external_diff_no_fileno(self):
+        # Make sure that we can handle not having a fileno, even
+        # if the diff is large
+        lines = external_udiff_lines(['boo\n']*10000,
+                                     ['goo\n']*10000,
+                                     use_stringio=True)
+        self.check_patch(lines)
+        
     def test_internal_diff_default(self):
         # Default internal diff encoding is utf8
         output = StringIO()

=== added file 'bzrlib/tests/test_emptytree.py'
--- bzrlib/tests/test_emptytree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_emptytree.py	2006-06-16 15:25:15 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the EmptyTree class."""
+
+import bzrlib
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tree import EmptyTree
+
+
+class TestTreeWithCommits(TestCaseWithTransport):
+
+    def test_empty_no_unknowns(self):
+        self.assertEqual([], list(EmptyTree().unknowns()))
+
+    def test_no_conflicts(self):
+        self.assertEqual([], list(EmptyTree().conflicts()))
+
+    def test_parents(self):
+        self.assertEqual([], EmptyTree().get_parent_ids())

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2006-02-26 07:54:02 +0000
+++ bzrlib/tests/test_errors.py	2006-06-15 05:36:34 +0000
@@ -27,8 +27,8 @@
     def test_no_repo(self):
         dir = bzrdir.BzrDir.create(self.get_url())
         error = errors.NoRepositoryPresent(dir)
-        self.assertNotEqual(-1, str(error).find(repr(dir.transport.clone('..').base)))
-        self.assertEqual(-1, str(error).find(repr(dir.transport.base)))
+        self.assertNotEqual(-1, str(error).find((dir.transport.clone('..').base)))
+        self.assertEqual(-1, str(error).find((dir.transport.base)))
 
     def test_up_to_date(self):
         error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2006-06-06 16:16:09 +0000
+++ bzrlib/tests/test_msgeditor.py	2006-06-10 13:58:01 +0000
@@ -102,7 +102,7 @@
             f = file('fed.bat', 'w')
             f.write("""\
 @echo off
-%s fed.py %%1
+"%s" fed.py %%1
 """ % sys.executable)
             f.close()
             os.environ['BZR_EDITOR'] = 'fed.bat'
@@ -123,7 +123,7 @@
         working_tree = self.make_uncommitted_tree()
 
         if sys.platform == 'win32':
-            os.environ['BZR_EDITOR'] = 'del'
+            os.environ['BZR_EDITOR'] = 'cmd.exe /c del'
         else:
             os.environ['BZR_EDITOR'] = 'rm'
 

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2006-03-22 17:48:49 +0000
+++ bzrlib/tests/test_options.py	2006-06-20 03:30:14 +0000
@@ -15,8 +15,6 @@
         eq = self.assertEquals
         eq(parse_args(cmd_commit(), ['--help']),
            ([], {'help': True}))
-        eq(parse_args(cmd_status(), ['--all']),
-           ([], {'all': True}))
         eq(parse_args(cmd_commit(), ['--message=biter']),
            ([], {'message': 'biter'}))
         ## eq(parse_args(cmd_log(),  '-r 500'.split()),
@@ -46,7 +44,7 @@
 
     def test_unknown_short_opt(self):
         out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
-        self.assertContainsRe(err, r'unknown short option')
+        self.assertContainsRe(err, r'unknown option')
 
 
 #     >>> parse_args('log -r 500'.split())

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-06-10 19:32:47 +0000
+++ bzrlib/tests/test_osutils.py	2006-06-18 18:56:23 +0000
@@ -25,7 +25,12 @@
 import bzrlib
 from bzrlib.errors import BzrBadParameterNotUnicode, InvalidURL
 import bzrlib.osutils as osutils
-from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
+from bzrlib.tests import (
+        StringIOWrapper,
+        TestCase, 
+        TestCaseInTempDir, 
+        TestSkipped,
+        )
 
 
 class TestOSUtils(TestCaseInTempDir):
@@ -266,3 +271,118 @@
         self.assertEqual(expected_dirblocks[1:],
             [[line[0:3] for line in block] for block in result])
 
+    def assertPathCompare(self, path_less, path_greater):
+        """check that path_less and path_greater compare correctly."""
+        self.assertEqual(0, osutils.compare_paths_prefix_order(
+            path_less, path_less))
+        self.assertEqual(0, osutils.compare_paths_prefix_order(
+            path_greater, path_greater))
+        self.assertEqual(-1, osutils.compare_paths_prefix_order(
+            path_less, path_greater))
+        self.assertEqual(1, osutils.compare_paths_prefix_order(
+            path_greater, path_less))
+
+    def test_compare_paths_prefix_order(self):
+        # root before all else
+        self.assertPathCompare("/", "/a")
+        # alpha within a dir
+        self.assertPathCompare("/a", "/b")
+        self.assertPathCompare("/b", "/z")
+        # high dirs before lower.
+        self.assertPathCompare("/z", "/a/a")
+        # except if the deeper dir should be output first
+        self.assertPathCompare("/a/b/c", "/d/g")
+        # lexical betwen dirs of the same height
+        self.assertPathCompare("/a/z", "/z/z")
+        self.assertPathCompare("/a/c/z", "/a/d/e")
+
+        # this should also be consistent for no leading / paths
+        # root before all else
+        self.assertPathCompare("", "a")
+        # alpha within a dir
+        self.assertPathCompare("a", "b")
+        self.assertPathCompare("b", "z")
+        # high dirs before lower.
+        self.assertPathCompare("z", "a/a")
+        # except if the deeper dir should be output first
+        self.assertPathCompare("a/b/c", "d/g")
+        # lexical betwen dirs of the same height
+        self.assertPathCompare("a/z", "z/z")
+        self.assertPathCompare("a/c/z", "a/d/e")
+
+    def test_path_prefix_sorting(self):
+        """Doing a sort on path prefix should match our sample data."""
+        original_paths = [
+            'a',
+            'a/b',
+            'a/b/c',
+            'b',
+            'b/c',
+            'd',
+            'd/e',
+            'd/e/f',
+            'd/f',
+            'd/g',
+            'g',
+            ]
+
+        dir_sorted_paths = [
+            'a',
+            'b',
+            'd',
+            'g',
+            'a/b',
+            'a/b/c',
+            'b/c',
+            'd/e',
+            'd/f',
+            'd/g',
+            'd/e/f',
+            ]
+
+        self.assertEqual(
+            dir_sorted_paths,
+            sorted(original_paths, key=osutils.path_prefix_key))
+        # using the comparison routine shoudl work too:
+        self.assertEqual(
+            dir_sorted_paths,
+            sorted(original_paths, cmp=osutils.compare_paths_prefix_order))
+
+
+class TestTerminalEncoding(TestCase):
+    """Test the auto-detection of proper terminal encoding."""
+
+    def setUp(self):
+        self._stdout = sys.stdout
+        self._stderr = sys.stderr
+        self._stdin = sys.stdin
+        self._user_encoding = bzrlib.user_encoding
+
+        self.addCleanup(self._reset)
+
+        sys.stdout = StringIOWrapper()
+        sys.stdout.encoding = 'stdout_encoding'
+        sys.stderr = StringIOWrapper()
+        sys.stderr.encoding = 'stderr_encoding'
+        sys.stdin = StringIOWrapper()
+        sys.stdin.encoding = 'stdin_encoding'
+        bzrlib.user_encoding = 'user_encoding'
+
+    def _reset(self):
+        sys.stdout = self._stdout
+        sys.stderr = self._stderr
+        sys.stdin = self._stdin
+        bzrlib.user_encoding = self._user_encoding
+
+    def test_get_terminal_encoding(self):
+        # first preference is stdout encoding
+        self.assertEqual('stdout_encoding', osutils.get_terminal_encoding())
+
+        sys.stdout.encoding = None
+        # if sys.stdout is None, fall back to sys.stdin
+        self.assertEqual('stdin_encoding', osutils.get_terminal_encoding())
+
+        sys.stdin.encoding = None
+        # and in the worst case, use bzrlib.user_encoding
+        self.assertEqual('user_encoding', osutils.get_terminal_encoding())
+

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2005-12-16 03:43:14 +0000
+++ bzrlib/tests/test_plugins.py	2006-06-18 23:19:23 +0000
@@ -22,9 +22,12 @@
 # comments.
 
 import os
+from StringIO import StringIO
 
 import bzrlib.plugin
 import bzrlib.plugins
+import bzrlib.commands
+import bzrlib.help
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.osutils import pathjoin, abspath
 
@@ -137,3 +140,57 @@
             if getattr(bzrlib.plugins, 'plugin', None):
                 del bzrlib.plugins.plugin
         self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+
+
+class TestPluginHelp(TestCaseInTempDir):
+
+    def split_help_commands(self):
+        help = {}
+        current = None
+        for line in self.capture('help commands').splitlines():
+            if line.startswith('bzr '):
+                current = line.split()[1]
+            help[current] = help.get(current, '') + line
+
+        return help
+
+    def test_plugin_help_builtins_unaffected(self):
+        # Check we don't get false positives
+        help_commands = self.split_help_commands()
+        for cmd_name in bzrlib.commands.builtin_command_names():
+            if cmd_name in bzrlib.commands.plugin_command_names():
+                continue
+            help = StringIO()
+            try:
+                bzrlib.help.help_on_command(cmd_name, help)
+            except NotImplementedError:
+                # some commands have no help
+                pass
+            else:
+                help.seek(0)
+                self.assertNotContainsRe(help.read(), 'From plugin "[^"]*"')
+
+            if help in help_commands.keys():
+                # some commands are hidden
+                help = help_commands[cmd_name]
+                self.assertNotContainsRe(help, 'From plugin "[^"]*"')
+
+    def test_plugin_help_shows_plugin(self):
+        # Create a test plugin
+        os.mkdir('plugin_test')
+        f = open(pathjoin('plugin_test', 'myplug.py'), 'w')
+        f.write(PLUGIN_TEXT)
+        f.close()
+
+        try:
+            # Check its help
+            bzrlib.plugin.load_from_dirs(['plugin_test'])
+            bzrlib.commands.register_command( bzrlib.plugins.myplug.cmd_myplug)
+            help = self.capture('help myplug')
+            self.assertContainsRe(help, 'From plugin "myplug"')
+            help = self.split_help_commands()['myplug']
+            self.assertContainsRe(help, 'From plugin "myplug"')
+        finally:
+            # remove the plugin 'plugin'
+            if getattr(bzrlib.plugins, 'plugin', None):
+                del bzrlib.plugins.plugin

=== added file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_read_bundle.py	2006-06-15 21:24:47 +0000
@@ -0,0 +1,117 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test read_bundle works properly across various transports."""
+
+import cStringIO
+import os
+
+import bzrlib.bundle
+from bzrlib.bundle.serializer import write_bundle
+import bzrlib.bzrdir
+import bzrlib.errors as errors
+from bzrlib.tests import TestCaseInTempDir
+import bzrlib.transport
+from bzrlib.transport.memory import MemoryTransport
+import bzrlib.urlutils
+
+
+# jam 20060615 Originally I thought I should use TestCaseWithTransport
+#       but it turns out that doesn't play well with the transport adapter
+class TestReadBundleFromURL(TestCaseInTempDir):
+    """Test that read_bundle works properly across multiple transports"""
+
+    def setUp(self):
+        super(TestReadBundleFromURL, self).setUp()
+        self._server = self.transport_server()
+        self._server.setUp()
+        self._transport = None
+
+    def tearDown(self):
+        self._transport = None
+        self._server.tearDown()
+        super(TestReadBundleFromURL, self).tearDown()
+        
+    def get_transport(self):
+        """Return a connected transport to the local directory."""
+        base_url = self._server.get_url()
+        t = bzrlib.transport.get_transport(base_url)
+        if not isinstance(t, self.transport_class):
+            # we want to make sure to construct one particular class, even if
+            # there are several available implementations of this transport;
+            # therefore construct it by hand rather than through the regular
+            # get_transport method
+            t = self.transport_class(base_url)
+        return t
+
+    def get_url(self, relpath=''):
+        return bzrlib.urlutils.join(self._server.get_url(), relpath)
+
+    def create_test_bundle(self):
+        # Can't use self.get_transport() because that asserts that 
+        # it is not readonly, so just skip tests where the server is readonly
+        self._transport = self.get_transport()
+        #if isinstance(self._transport, MemoryTransport):
+        #    return None
+        self.build_tree(['tree/', 'tree/a', 'tree/subdir/'])
+
+        format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+
+
+        bzrdir = format.initialize('tree')
+        repo = bzrdir.create_repository()
+        branch = repo.bzrdir.create_branch()
+        wt = branch.bzrdir.create_workingtree()
+
+        wt.add(['a', 'subdir/'])
+        wt.commit('new project', rev_id='commit-1')
+
+        out = cStringIO.StringIO()
+        rev_ids = write_bundle(wt.branch.repository,
+                               wt.last_revision(), None, out)
+        out.seek(0)
+        if self._transport.is_readonly():
+            f = open('test_bundle', 'wb')
+            f.write(out.getvalue())
+            f.close()
+        else:
+            self._transport.put('test_bundle', out)
+            self.log('Put to: %s', self.get_url('test_bundle'))
+        return wt
+
+    def test_read_bundle_from_url(self):
+        wt = self.create_test_bundle()
+        if wt is None:
+            return
+
+        reader = bzrlib.bundle.read_bundle_from_url(
+                    self.get_url('test_bundle'))
+        info, bundle_tree = reader.get_bundle(wt.branch.repository)
+        self.assertEqual('commit-1', bundle_tree.revision_id)
+
+    def test_read_fail(self):
+        # Trying to read from a directory, or non-bundle file
+        # should fail with NotABundle
+        wt = self.create_test_bundle()
+        if wt is None:
+            return
+
+        self.assertRaises(errors.NotABundle, 
+            bzrlib.bundle.read_bundle_from_url, 
+            self.get_url('tree'))
+        self.assertRaises(errors.NotABundle, 
+            bzrlib.bundle.read_bundle_from_url, 
+            self.get_url('tree/a'))

=== modified file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	2006-02-25 05:48:54 +0000
+++ bzrlib/tests/test_reconcile.py	2006-06-16 15:25:15 +0000
@@ -23,7 +23,6 @@
 from bzrlib.revision import Revision
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 from bzrlib.transport import get_transport
-from bzrlib.tree import EmptyTree
 from bzrlib.workingtree import WorkingTree
 
 

=== added file 'bzrlib/tests/test_revisiontree.py'
--- bzrlib/tests/test_revisiontree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_revisiontree.py	2006-06-16 15:25:15 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the RevisionTree class."""
+
+import bzrlib
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tree import RevisionTree
+
+
+class TestTreeWithCommits(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestTreeWithCommits, self).setUp()
+        self.t = self.make_branch_and_tree('.')
+        self.rev_id = self.t.commit('foo', allow_pointless=True)
+        self.rev_tree = self.t.branch.repository.revision_tree(self.rev_id)
+
+    def test_empty_no_unknowns(self):
+        self.assertEqual([], list(self.rev_tree.unknowns()))
+
+    def test_no_conflicts(self):
+        self.assertEqual([], list(self.rev_tree.conflicts()))
+
+    def test_parents(self):
+        """RevisionTree.parent_ids should match the revision graph."""
+        # XXX: TODO: Should this be a repository_implementation test ?
+        # at the end of the graph, we get []
+        self.assertEqual([], self.rev_tree.get_parent_ids())
+        # do a commit to look further up
+        revid_2 = self.t.commit('bar', allow_pointless=True)
+        self.assertEqual(
+            [self.rev_id],
+            self.t.branch.repository.revision_tree(revid_2).get_parent_ids())
+        # TODO commit a merge and check it is reported correctly.
+        

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/test_source.py	2006-06-20 05:32:16 +0000
@@ -72,4 +72,4 @@
         # written. Note that this is an exact equality so that when the number
         # drops, it is not given a buffer but rather this test updated
         # immediately.
-        self.assertEqual(3, occurences)
+        self.assertEqual(2, occurences)

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2005-12-27 22:56:50 +0000
+++ bzrlib/tests/test_trace.py	2006-06-20 03:30:14 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,47 +18,64 @@
 
 """Tests for trace library"""
 
+import errno
 import os
 import sys
+from StringIO import StringIO
 
 from bzrlib.tests import TestCaseInTempDir, TestCase
-from bzrlib.trace import format_exception_short, mutter
-from bzrlib.errors import NotBranchError, BzrError, BzrNewError
+from bzrlib.trace import mutter, report_exception
+from bzrlib.errors import NotBranchError
+
+
+def _format_exception():
+    """Format an exception as it would normally be displayed to the user"""
+    buf = StringIO()
+    report_exception(sys.exc_info(), buf)
+    return buf.getvalue()
+
 
 class TestTrace(TestCase):
+
     def test_format_sys_exception(self):
-        """Short formatting of exceptions"""
         try:
             raise NotImplementedError, "time travel"
         except NotImplementedError:
             pass
-        error_lines = format_exception_short(sys.exc_info()).splitlines()
-        self.assertEqualDiff(error_lines[0], 
-                'exceptions.NotImplementedError: time travel')
-        self.assertContainsRe(error_lines[1], 
-                r'^  at .*trace\.py line \d+$')  
-        self.assertContainsRe(error_lines[2], 
-                r'^  in test_format_sys_exception$')
+        err = _format_exception()
+        self.assertEqualDiff(err.splitlines()[0],
+                'bzr: ERROR: exceptions.NotImplementedError: time travel')
+        self.assertContainsRe(err,
+                r'File.*test_trace.py')
+
+    def test_format_interrupt_exception(self):
+        try:
+            raise KeyboardInterrupt()
+        except KeyboardInterrupt:
+            # XXX: Some risk that a *real* keyboard interrupt won't be seen
+            pass
+        msg = _format_exception()
+        self.assertTrue(len(msg) > 0)
+        self.assertEqualDiff(msg, 'bzr: interrupted\n')
+
+    def test_format_os_error(self):
+        try:
+            file('nosuchfile22222')
+        except (OSError, IOError):
+            pass
+        msg = _format_exception()
+        self.assertContainsRe(msg, r'^bzr: ERROR: \[Errno .*\] No such file.*nosuchfile')
+
 
     def test_format_exception(self):
-        """Short formatting of exceptions"""
+        """Short formatting of bzr exceptions"""
         try:
             raise NotBranchError, 'wibble'
         except NotBranchError:
             pass
-        msg = format_exception_short(sys.exc_info())
-        self.assertEqualDiff(msg, 'Not a branch: wibble')
-
-    def test_format_old_exception(self):
-        # format a class that doesn't descend from BzrNewError; 
-        # remove this test when everything is unified there
-        self.assertFalse(issubclass(BzrError, BzrNewError))
-        try:
-            raise BzrError('some old error')
-        except BzrError:
-            pass
-        msg = format_exception_short(sys.exc_info())
-        self.assertEqualDiff(msg, 'some old error')
+        msg = _format_exception()
+        self.assertTrue(len(msg) > 0)
+        self.assertEqualDiff(msg, 'bzr: ERROR: Not a branch: wibble\n')
 
     def test_trace_unicode(self):
         """Write Unicode to trace log"""
@@ -67,6 +83,15 @@
         self.assertContainsRe('the unicode character',
                 self._get_log())
 
+    def test_report_broken_pipe(self):
+        try:
+            raise IOError(errno.EPIPE, 'broken pipe foofofo')
+        except IOError, e:
+            msg = _format_exception()
+            self.assertEquals(msg, "bzr: broken pipe\n")
+        else:
+            self.fail("expected error not raised")
+
     def test_mutter_never_fails(self):
         # Even if the decode/encode stage fails, mutter should not
         # raise an exception
@@ -81,4 +106,3 @@
         self.assertContainsRe(log, 'Writing a greek mu')
         self.assertContainsRe(log, 'UnicodeError')
         self.assertContainsRe(log, "'But fails in an ascii string")
-

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-06-07 11:06:23 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-06-18 18:56:23 +0000
@@ -700,8 +700,8 @@
         except NotImplementedError:
             raise TestSkipped("Transport %s has no bogus URL support." %
                               self._server.__class__)
-        t = bzrlib.transport.get_transport(url)
         try:
+            t = bzrlib.transport.get_transport(url)
             t.get('.bzr/branch')
         except (ConnectionError, NoSuchFile), e:
             pass
@@ -922,7 +922,7 @@
                 ]
 
         try:
-            self.build_tree(files, transport=t)
+            self.build_tree(files, transport=t, line_endings='binary')
         except UnicodeError:
             raise TestSkipped("cannot handle unicode paths in current encoding")
 

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-06-07 16:15:45 +0000
+++ bzrlib/tests/test_urlutils.py	2006-06-18 18:56:23 +0000
@@ -91,7 +91,9 @@
         norm_file('uni/%25C2%25B5', u'uni/%C2%B5')
         norm_file('uni/%20b', u'uni/ b')
         # All the crazy characters get escaped in local paths => file:/// urls
-        norm_file('%27%3B/%3F%3A%40%26%3D%2B%24%2C%23%20', "';/?:@&=+$,# ")
+        # The ' ' character must not be at the end, because on win32
+        # it gets stripped off by ntpath.abspath
+        norm_file('%27%20%3B/%3F%3A%40%26%3D%2B%24%2C%23', "' ;/?:@&=+$,#")
 
     def test_normalize_url_hybrid(self):
         # Anything with a scheme:// should be treated as a hybrid url
@@ -209,6 +211,12 @@
         test('http://bar', 'http://foo', 'http://bar')
         test('sftp://bzr/foo', 'http://foo', 'bar', 'sftp://bzr/foo')
         test('file:///bar', 'foo', 'file:///bar')
+
+        # From a base path
+        test('file:///foo', 'file:///', 'foo')
+        test('file:///bar/foo', 'file:///bar/', 'foo')
+        test('http://host/foo', 'http://host/', 'foo')
+        test('http://host/', 'http://host', '')
         
         # Invalid joinings
         # Cannot go above root
@@ -248,29 +256,43 @@
 
     def test_win32_local_path_to_url(self):
         to_url = urlutils._win32_local_path_to_url
-        self.assertEqual('file:///C:/path/to/foo',
+        self.assertEqual('file:///c:/path/to/foo',
             to_url('C:/path/to/foo'))
+        # BOGUS: on win32, ntpath.abspath will strip trailing
+        #       whitespace, so this will always fail
+        #       Though under linux, it fakes abspath support
+        #       and thus will succeed
+        # self.assertEqual('file:///C:/path/to/foo%20',
+        #     to_url('C:/path/to/foo '))
+        self.assertEqual('file:///c:/path/to/f%20oo',
+            to_url('C:/path/to/f oo'))
 
         try:
             result = to_url(u'd:/path/to/r\xe4ksm\xf6rg\xe5s')
         except UnicodeError:
             raise TestSkipped("local encoding cannot handle unicode")
 
-        self.assertEqual('file:///D:/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s', result)
+        self.assertEqual('file:///d:/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s', result)
 
     def test_win32_local_path_from_url(self):
         from_url = urlutils._win32_local_path_from_url
-        self.assertEqual('C:/path/to/foo',
+        self.assertEqual('c:/path/to/foo',
             from_url('file:///C|/path/to/foo'))
-        self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
+        self.assertEqual(u'd:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d|/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s'))
-        self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
+        self.assertEqual(u'd:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d:/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
 
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
         self.assertRaises(InvalidURL, from_url, 'file:///path/to/foo')
 
+    def test__win32_extract_drive_letter(self):
+        extract = urlutils._win32_extract_drive_letter
+        self.assertEqual(('file:///C:', '/foo'), extract('file://', '/C:/foo'))
+        self.assertEqual(('file:///d|', '/path'), extract('file://', '/d|/path'))
+        self.assertRaises(InvalidURL, extract, 'file://', '/path')
+
     def test_split(self):
         # Test bzrlib.urlutils.split()
         split = urlutils.split
@@ -306,6 +328,16 @@
         self.assertEqual(('path/..', 'foo'), split('path/../foo'))
         self.assertEqual(('../path', 'foo'), split('../path/foo'))
 
+    def test__win32_strip_local_trailing_slash(self):
+        strip = urlutils._win32_strip_local_trailing_slash
+        self.assertEqual('file://', strip('file://'))
+        self.assertEqual('file:///', strip('file:///'))
+        self.assertEqual('file:///C', strip('file:///C'))
+        self.assertEqual('file:///C:', strip('file:///C:'))
+        self.assertEqual('file:///d|', strip('file:///d|'))
+        self.assertEqual('file:///C:/', strip('file:///C:/'))
+        self.assertEqual('file:///C:/a', strip('file:///C:/a/'))
+
     def test_strip_trailing_slash(self):
         sts = urlutils.strip_trailing_slash
         if sys.platform == 'win32':
@@ -348,8 +380,8 @@
 
         test('http://foo', 'http://foo')
         if sys.platform == 'win32':
-            test('C:/foo/path', 'file:///C|/foo/path')
-            test('C:/foo/path', 'file:///C:/foo/path')
+            test('c:/foo/path', 'file:///C|/foo/path')
+            test('c:/foo/path', 'file:///C:/foo/path')
         else:
             test('/foo/path', 'file:///foo/path')
 

=== modified file 'bzrlib/tests/workingtree_implementations/__init__.py'
--- bzrlib/tests/workingtree_implementations/__init__.py	2006-05-18 10:31:23 +0000
+++ bzrlib/tests/workingtree_implementations/__init__.py	2006-06-16 15:25:15 +0000
@@ -54,6 +54,7 @@
         'bzrlib.tests.workingtree_implementations.test_basis_inventory',
         'bzrlib.tests.workingtree_implementations.test_break_lock',
         'bzrlib.tests.workingtree_implementations.test_commit',
+        'bzrlib.tests.workingtree_implementations.test_get_parent_ids',
         'bzrlib.tests.workingtree_implementations.test_is_control_filename',
         'bzrlib.tests.workingtree_implementations.test_is_ignored',
         'bzrlib.tests.workingtree_implementations.test_pull',

=== modified file 'bzrlib/tests/workingtree_implementations/test_commit.py'
--- bzrlib/tests/workingtree_implementations/test_commit.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/workingtree_implementations/test_commit.py	2006-06-20 05:32:16 +0000
@@ -82,8 +82,16 @@
 
     def test_commit_sets_last_revision(self):
         tree = self.make_branch_and_tree('tree')
-        tree.commit('foo', rev_id='foo', allow_pointless=True)
+        committed_id = tree.commit('foo', rev_id='foo', allow_pointless=True)
         self.assertEqual('foo', tree.last_revision())
+        # the commit should have returned the same id we asked for.
+        self.assertEqual('foo', committed_id)
+
+    def test_commit_returns_revision_id(self):
+        tree = self.make_branch_and_tree('.')
+        committed_id = tree.commit('message', allow_pointless=True)
+        self.assertTrue(tree.branch.repository.has_revision(committed_id))
+        self.assertNotEqual(None, committed_id)
 
     def test_commit_local_unbound(self):
         # using the library api to do a local commit on unbound branches is 

=== added file 'bzrlib/tests/workingtree_implementations/test_get_parent_ids.py'
--- bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	2006-06-16 15:25:15 +0000
@@ -0,0 +1,34 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for interface conformance of 'workingtree.get_parent_ids'"""
+
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestGetParentIds(TestCaseWithWorkingTree):
+
+    def test_get_parent_ids(self):
+        t = self.make_branch_and_tree('t1')
+        self.assertEqual([], t.get_parent_ids())
+        rev1_id = t.commit('foo', allow_pointless=True)
+        self.assertEqual([rev1_id], t.get_parent_ids())
+        t2 = t.bzrdir.sprout('t2').open_workingtree()
+        rev2_id = t2.commit('foo', allow_pointless=True)
+        self.assertEqual([rev2_id], t2.get_parent_ids())
+        self.merge(t2.branch, t)
+        self.assertEqual([rev1_id, rev2_id], t.get_parent_ids())

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2006-06-20 03:57:11 +0000
+++ bzrlib/trace.py	2006-06-20 05:32:16 +0000
@@ -1,4 +1,18 @@
-# Copyright (C) 2005, Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """Messages and logging for bazaar-ng.
 
@@ -32,21 +46,21 @@
 form.
 """
 
-# TODO: in debug mode, stderr should get full tracebacks and also
-# debug messages.  (Is this really needed?)
-
 # FIXME: Unfortunately it turns out that python's logging module
 # is quite expensive, even when the message is not printed by any handlers.
 # We should perhaps change back to just simply doing it here.
 
 
+import errno
+import os
 import sys
-import os
 import logging
 
 import bzrlib
 from bzrlib.errors import BzrError, BzrNewError
-
+from bzrlib.symbol_versioning import (deprecated_function,
+        zero_nine,
+        )
 
 _file_handler = None
 _stderr_handler = None
@@ -56,25 +70,6 @@
 _bzr_log_file = None
 
 
-class QuietFormatter(logging.Formatter):
-    """Formatter that suppresses the details of errors.
-
-    This is used by default on stderr so as not to scare the user.
-    """
-    # At first I tried overriding FormatException to suppress the
-    # exception details, but that has global effect: no loggers
-    # can get the exception details is we suppress them here.
-
-    def format(self, record):
-        if record.levelno >= logging.WARNING:
-            s = 'bzr: ' + record.levelname + ': '
-        else:
-            s = ''
-        s += record.getMessage()
-        if record.exc_info:
-            s += '\n' + format_exception_short(record.exc_info)
-        return s
-        
 # configure convenient aliases for output routines
 
 _bzr_logger = logging.getLogger('bzr')
@@ -157,15 +152,7 @@
         warning("failed to open trace file: %s" % (e))
 
 
-def log_startup(argv):
-    debug('\n\nbzr %s invoked on python %s (%s)',
-          bzrlib.__version__,
-          '.'.join(map(str, sys.version_info)),
-          sys.platform)
-    debug('  arguments: %r', argv)
-    debug('  working dir: %r', os.getcwdu())
-
-
+@deprecated_function(zero_nine)
 def log_exception(msg=None):
     """Log the last exception to stderr and the trace file.
 
@@ -196,7 +183,6 @@
     # FIXME: if this is run twice, things get confused
     global _stderr_handler, _file_handler, _trace_file, _bzr_log_file
     _stderr_handler = logging.StreamHandler()
-    _stderr_handler.setFormatter(QuietFormatter())
     logging.getLogger('').addHandler(_stderr_handler)
     _stderr_handler.setLevel(logging.INFO)
     if not _file_handler:
@@ -204,8 +190,7 @@
     _trace_file = _bzr_log_file
     if _file_handler:
         _file_handler.setLevel(logging.DEBUG)
-    _bzr_logger.setLevel(logging.DEBUG) 
-
+    _bzr_logger.setLevel(logging.DEBUG)
 
 
 def be_quiet(quiet=True):
@@ -268,32 +253,40 @@
         enable_default_logging()
 
 
-def format_exception_short(exc_info):
-    """Make a short string form of an exception.
-
-    This is used for display to stderr.  It specially handles exception
-    classes without useful string methods.
-
-    The result has no trailing newline.
-
-    exc_info - typically an exception from sys.exc_info()
-    """
-    exc_type, exc_object, exc_tb = exc_info
-    try:
-        if exc_type is None:
-            return '(no exception)'
-        if isinstance(exc_object, (BzrError, BzrNewError)):
-            return str(exc_object)
-        else:
-            import traceback
-            tb = traceback.extract_tb(exc_tb)
-            msg = '%s: %s' % (exc_type, exc_object)
-            if msg[-1] == '\n':
-                msg = msg[:-1]
-            if tb:
-                msg += '\n  at %s line %d\n  in %s' % (tb[-1][:3])
-            return msg
-    except Exception, formatting_exc:
-        # XXX: is this really better than just letting it run up?
-        return '(error formatting exception of type %s: %s)' \
-                % (exc_type, formatting_exc)
+def report_exception(exc_info, err_file):
+    exc_type, exc_object, exc_tb = exc_info
+    if (isinstance(exc_object, IOError)
+        and getattr(exc_object, 'errno', None) == errno.EPIPE):
+        print >>err_file, "bzr: broken pipe"
+    elif isinstance(exc_object, KeyboardInterrupt):
+        print >>err_file, "bzr: interrupted"
+    elif getattr(exc_object, 'is_user_error', False):
+        report_user_error(exc_info, err_file)
+    elif isinstance(exc_object, (OSError, IOError)):
+        # Might be nice to catch all of these and show them as something more
+        # specific, but there are too many cases at the moment.
+        report_user_error(exc_info, err_file)
+    else:
+        report_bug(exc_info, err_file)
+
+
+# TODO: Should these be specially encoding the output?
+def report_user_error(exc_info, err_file):
+    print >>err_file, "bzr: ERROR:", str(exc_info[1])
+
+
+def report_bug(exc_info, err_file):
+    """Report an exception that probably indicates a bug in bzr"""
+    import traceback
+    exc_type, exc_object, exc_tb = exc_info
+    print >>err_file, "bzr: ERROR: %s: %s" % (exc_type, exc_object)
+    print >>err_file
+    traceback.print_exception(exc_type, exc_object, exc_tb, file=err_file)
+    print >>err_file
+    print >>err_file, 'bzr %s on python %s (%s)' % \
+                       (bzrlib.__version__,
+                        '.'.join(map(str, sys.version_info)),
+                        sys.platform)
+    print >>err_file, 'arguments: %r' % sys.argv
+    print >>err_file
+    print >>err_file, "** please send this report to bazaar-ng@lists.ubuntu.com"

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-06-20 03:57:11 +0000
+++ bzrlib/transport/__init__.py	2006-06-20 05:32:16 +0000
@@ -706,7 +706,7 @@
             raise errors.InvalidURL(base, error_str % m.group('proto'))
         # This doesn't look like a protocol, consider it a local path
         new_base = urlutils.local_path_to_url(base)
-        mutter('converting os path %r => url %s' , base, new_base)
+        mutter('converting os path %r => url %s', base, new_base)
         return new_base
 
     # Catch any URLs which are passing Unicode rather than ASCII

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-04-25 20:18:04 +0000
+++ bzrlib/transport/http/__init__.py	2006-06-11 03:40:17 +0000
@@ -19,11 +19,13 @@
 There are separate implementation modules for each http client implementation.
 """
 
+from collections import deque
+from cStringIO import StringIO
 import errno
 import os
-from collections import deque
-from cStringIO import StringIO
+import posixpath
 import re
+import sys
 import urlparse
 import urllib
 from warnings import warn
@@ -398,6 +400,34 @@
         method = getattr(self, mname)
         method()
 
+    if sys.platform == 'win32':
+        # On win32 you cannot access non-ascii filenames without
+        # decoding them into unicode first.
+        # However, under Linux, you can access bytestream paths
+        # without any problems. If this function was always active
+        # it would probably break tests when LANG=C was set
+        def translate_path(self, path):
+            """Translate a /-separated PATH to the local filename syntax.
+
+            For bzr, all url paths are considered to be utf8 paths.
+            On Linux, you can access these paths directly over the bytestream
+            request, but on win32, you must decode them, and access them
+            as Unicode files.
+            """
+            # abandon query parameters
+            path = urlparse.urlparse(path)[2]
+            path = posixpath.normpath(urllib.unquote(path))
+            path = path.decode('utf-8')
+            words = path.split('/')
+            words = filter(None, words)
+            path = os.getcwdu()
+            for word in words:
+                drive, word = os.path.splitdrive(word)
+                head, word = os.path.split(word)
+                if word in (os.curdir, os.pardir): continue
+                path = os.path.join(path, word)
+            return path
+
 
 class TestingHTTPServer(BaseHTTPServer.HTTPServer):
     def __init__(self, server_address, RequestHandlerClass, test_case):

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-06-09 09:57:44 +0000
+++ bzrlib/transport/local.py	2006-06-11 01:38:11 +0000
@@ -51,6 +51,7 @@
             base = base + '/'
         super(LocalTransport, self).__init__(base)
         self._local_base = urlutils.local_path_from_url(base)
+        ## mutter("_local_base: %r => %r", base, self._local_base)
 
     def should_cache(self):
         return False

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-06-07 11:06:23 +0000
+++ bzrlib/transport/sftp.py	2006-06-18 15:53:41 +0000
@@ -22,6 +22,7 @@
 import os
 import random
 import re
+import select
 import stat
 import subprocess
 import sys
@@ -302,6 +303,7 @@
             # What specific errors should we catch here?
             pass
 
+
 class SFTPTransport (Transport):
     """
     Transport implementation for SFTP access.
@@ -702,7 +704,11 @@
         vendor = _get_ssh_vendor()
         if vendor == 'loopback':
             sock = socket.socket()
-            sock.connect((self._host, self._port))
+            try:
+                sock.connect((self._host, self._port))
+            except socket.error, e:
+                raise ConnectionError('Unable to connect to SSH host %s:%s: %s'
+                                      % (self._host, self._port, e))
             self._sftp = SFTPClient(LoopbackSFTP(sock))
         elif vendor != 'none':
             sock = SFTPSubprocess(self._host, vendor, self._port,
@@ -723,8 +729,8 @@
             t.set_log_channel('bzr.paramiko')
             t.start_client()
         except paramiko.SSHException, e:
-            raise ConnectionError('Unable to reach SSH host %s:%d' %
-                                  (self._host, self._port), e)
+            raise ConnectionError('Unable to reach SSH host %s:%s: %s' 
+                                  % (self._host, self._port, e))
             
         server_key = t.get_remote_server_key()
         server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
@@ -883,9 +889,9 @@
 nvuQES5C9BMHjF39LZiGH1iLQy7FgdHyoP+eodI7
 -----END RSA PRIVATE KEY-----
 """
-    
-
-class SingleListener(threading.Thread):
+
+
+class SocketListener(threading.Thread):
 
     def __init__(self, callback):
         threading.Thread.__init__(self)
@@ -895,25 +901,39 @@
         self._socket.bind(('localhost', 0))
         self._socket.listen(1)
         self.port = self._socket.getsockname()[1]
-        self.stop_event = threading.Event()
-
-    def run(self):
-        s, _ = self._socket.accept()
-        # now close the listen socket
-        self._socket.close()
-        try:
-            self._callback(s, self.stop_event)
-        except socket.error:
-            pass #Ignore socket errors
-        except Exception, x:
-            # probably a failed test
-            warning('Exception from within unit test server thread: %r' % x)
+        self._stop_event = threading.Event()
 
     def stop(self):
-        self.stop_event.set()
+        # called from outside this thread
+        self._stop_event.set()
         # use a timeout here, because if the test fails, the server thread may
         # never notice the stop_event.
         self.join(5.0)
+        self._socket.close()
+
+    def run(self):
+        while True:
+            readable, writable_unused, exception_unused = \
+                select.select([self._socket], [], [], 0.1)
+            if self._stop_event.isSet():
+                return
+            if len(readable) == 0:
+                continue
+            try:
+                s, addr_unused = self._socket.accept()
+                # because the loopback socket is inline, and transports are
+                # never explicitly closed, best to launch a new thread.
+                threading.Thread(target=self._callback, args=(s,)).start()
+            except socket.error, x:
+                sys.excepthook(*sys.exc_info())
+                warning('Socket error during accept() within unit test server'
+                        ' thread: %r' % x)
+            except Exception, x:
+                # probably a failed test; unit test thread will log the
+                # failure/error
+                sys.excepthook(*sys.exc_info())
+                warning('Exception from within unit test server thread: %r' % 
+                        x)
 
 
 class SFTPServer(Server):
@@ -937,10 +957,12 @@
         """StubServer uses this to log when a new server is created."""
         self.logs.append(message)
 
-    def _run_server(self, s, stop_event):
+    def _run_server(self, s):
         ssh_server = paramiko.Transport(s)
         key_file = os.path.join(self._homedir, 'test_rsa.key')
-        file(key_file, 'w').write(STUB_SERVER_KEY)
+        f = open(key_file, 'w')
+        f.write(STUB_SERVER_KEY)
+        f.close()
         host_key = paramiko.RSAKey.from_private_key_file(key_file)
         ssh_server.add_server_key(host_key)
         server = StubServer(self)
@@ -950,7 +972,6 @@
         event = threading.Event()
         ssh_server.start_server(event, server)
         event.wait(5.0)
-        stop_event.wait(30.0)
     
     def setUp(self):
         global _ssh_vendor
@@ -961,7 +982,7 @@
             self._server_homedir = self._homedir
         self._root = '/'
         # FIXME WINDOWS: _root should be _server_homedir[0]:/
-        self._listener = SingleListener(self._run_server)
+        self._listener = SocketListener(self._run_server)
         self._listener.setDaemon(True)
         self._listener.start()
 
@@ -971,6 +992,13 @@
         self._listener.stop()
         _ssh_vendor = self._original_vendor
 
+    def get_bogus_url(self):
+        """See bzrlib.transport.Server.get_bogus_url."""
+        # this is chosen to try to prevent trouble with proxies, wierd dns,
+        # etc
+        return 'sftp://127.0.0.1:1/'
+
+
 
 class SFTPFullAbsoluteServer(SFTPServer):
     """A test server for sftp transports, using absolute urls and ssh."""
@@ -987,7 +1015,7 @@
         super(SFTPServerWithoutSSH, self).__init__()
         self._vendor = 'loopback'
 
-    def _run_server(self, sock, stop_event):
+    def _run_server(self, sock):
         class FakeChannel(object):
             def get_transport(self):
                 return self

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-06-12 01:53:19 +0000
+++ bzrlib/tree.py	2006-06-18 03:39:50 +0000
@@ -21,10 +21,11 @@
 from cStringIO import StringIO
 
 import bzrlib
-from bzrlib.trace import mutter, note
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import fingerprint_file
+import bzrlib.revision
+from bzrlib.trace import mutter, note
 
 class Tree(object):
     """Abstract file tree.
@@ -48,6 +49,22 @@
     trees or versioned trees.
     """
     
+    def conflicts(self):
+        """Get a list of the conflicts in the tree.
+
+        Each conflict is an instance of bzrlib.conflicts.Conflict.
+        """
+        return []
+
+    def get_parent_ids(self):
+        """Get the parent ids for this tree. 
+
+        :return: a list of parent ids. [] is returned to indicate
+        a tree with no parents.
+        :raises: BzrError if the parents are not known.
+        """
+        raise NotImplementedError(self.get_parent_ids)
+    
     def has_filename(self, filename):
         """True if the tree has given filename."""
         raise NotImplementedError()
@@ -108,6 +125,13 @@
     def lock_read(self):
         pass
 
+    def unknowns(self):
+        """What files are present in this tree and unknown.
+        
+        :return: an iterator over the unknown files.
+        """
+        return iter([])
+
     def unlock(self):
         pass
 
@@ -134,18 +158,31 @@
     """
     
     def __init__(self, branch, inv, revision_id):
-        self._branch = branch
+        # for compatability the 'branch' parameter has not been renamed to 
+        # repository at this point. However, we should change RevisionTree's
+        # construction to always be via Repository and not via direct 
+        # construction - this will mean that we can change the constructor
+        # with much less chance of breaking client code.
+        self._repository = branch
         self._weave_store = branch.weave_store
         self._inventory = inv
         self._revision_id = revision_id
 
+    def get_parent_ids(self):
+        """See Tree.get_parent_ids.
+
+        A RevisionTree's parents match the revision graph.
+        """
+        parent_ids = self._repository.get_revision(self._revision_id).parent_ids
+        return parent_ids
+        
     def get_revision_id(self):
         """Return the revision id associated with this tree."""
         return self._revision_id
 
     def get_weave(self, file_id):
         return self._weave_store.get_weave(file_id,
-                self._branch.get_transaction())
+                self._repository.get_transaction())
 
     def get_file_lines(self, file_id):
         ie = self._inventory[file_id]
@@ -169,7 +206,7 @@
 
     def get_file_mtime(self, file_id, path=None):
         ie = self._inventory[file_id]
-        revision = self._branch.get_revision(ie.revision)
+        revision = self._repository.get_revision(ie.revision)
         return revision.timestamp
 
     def is_executable(self, file_id, path=None):
@@ -194,16 +231,24 @@
         return self._inventory[file_id].kind
 
     def lock_read(self):
-        self._branch.lock_read()
+        self._repository.lock_read()
 
     def unlock(self):
-        self._branch.unlock()
+        self._repository.unlock()
 
 
 class EmptyTree(Tree):
+
     def __init__(self):
         self._inventory = Inventory()
 
+    def get_parent_ids(self):
+        """See Tree.get_parent_ids.
+
+        An EmptyTree always has NULL_REVISION as the only parent.
+        """
+        return []
+
     def get_symlink_target(self, file_id):
         return None
 

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2006-06-10 13:27:24 +0000
+++ bzrlib/urlutils.py	2006-06-18 18:56:23 +0000
@@ -19,7 +19,7 @@
 """A collection of function for handling URL operations."""
 
 import os
-from posixpath import split as _posix_split
+from posixpath import split as _posix_split, normpath as _posix_normpath
 import re
 import sys
 import urllib
@@ -116,6 +116,11 @@
     if m:
         scheme = m.group('scheme')
         path = m.group('path').split('/')
+        if path[-1:] == ['']:
+            # Strip off a trailing slash
+            # This helps both when we are at the root, and when
+            # 'base' has an extra slash at the end
+            path = path[:-1]
     else:
         path = base.split('/')
 
@@ -160,46 +165,54 @@
     """
     # importing directly from posixpath allows us to test this 
     # on non-posix platforms
-    from posixpath import normpath
-    return 'file://' + escape(normpath(bzrlib.osutils._posix_abspath(path)))
+    return 'file://' + escape(_posix_normpath(
+        bzrlib.osutils._posix_abspath(path)))
 
 
 def _win32_local_path_from_url(url):
-    """Convert a url like file:///C|/path/to/foo into C:/path/to/foo"""
+    """Convert a url like file:///C:/path/to/foo into C:/path/to/foo"""
     if not url.startswith('file:///'):
         raise errors.InvalidURL(url, 'local urls must start with file:///')
     # We strip off all 3 slashes
     win32_url = url[len('file:///'):]
-    if (win32_url[0] not in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
+    if (win32_url[0] not in ('abcdefghijklmnopqrstuvwxyz'
+                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
         or win32_url[1] not in  '|:'
         or win32_url[2] != '/'):
-        raise errors.InvalidURL(url, 'Win32 file urls start with file:///X|/, where X is a valid drive letter')
-    # TODO: jam 20060426, we could .upper() or .lower() the drive letter
-    #       for better consistency.
-    return win32_url[0].upper() + u':' + unescape(win32_url[2:])
+        raise errors.InvalidURL(url, 'Win32 file urls start with'
+                ' file:///x:/, where x is a valid drive letter')
+    # Preferentially using .lower() because os.getcwd() returns
+    # paths with lowercase drive letters, and that helps
+    # bzrlib.osutils.relpath() work correctly
+    return win32_url[0].lower() + u':' + unescape(win32_url[2:])
 
 
 def _win32_local_path_to_url(path):
-    """Convert a local path like ./foo into a URL like file:///C|/path/to/foo
+    """Convert a local path like ./foo into a URL like file:///C:/path/to/foo
 
     This also handles transforming escaping unicode characters, etc.
     """
     # importing directly from ntpath allows us to test this 
-    # on non-win32 platforms
+    # on non-win32 platform
+    # FIXME: It turns out that on nt, ntpath.abspath uses nt._getfullpathname
+    #       which actually strips trailing space characters.
+    #       The worst part is that under linux ntpath.abspath has different
+    #       semantics, since 'nt' is not an available module.
     win32_path = bzrlib.osutils._nt_normpath(
         bzrlib.osutils._win32_abspath(path)).replace('\\', '/')
-    return 'file:///' + win32_path[0].upper() + ':' + escape(win32_path[2:])
+    return 'file:///' + win32_path[0].lower() + ':' + escape(win32_path[2:])
 
 
 local_path_to_url = _posix_local_path_to_url
 local_path_from_url = _posix_local_path_from_url
 MIN_ABS_FILEURL_LENGTH = len('file:///')
+WIN32_MIN_ABS_FILEURL_LENGTH = len('file:///C:/')
 
 if sys.platform == 'win32':
     local_path_to_url = _win32_local_path_to_url
     local_path_from_url = _win32_local_path_from_url
 
-    MIN_ABS_FILEURL_LENGTH = len('file:///C|/')
+    MIN_ABS_FILEURL_LENGTH = WIN32_MIN_ABS_FILEURL_LENGTH
 
 
 _url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,})://(?P<path>.*)$')
@@ -291,6 +304,18 @@
     return "/".join(output_sections) or "."
 
 
+def _win32_extract_drive_letter(url_base, path):
+    """On win32 the drive letter needs to be added to the url base."""
+    # Strip off the drive letter
+    # path is currently /C:/foo
+    if len(path) < 3 or path[2] not in ':|' or path[3] != '/':
+        raise errors.InvalidURL(url_base + path, 
+            'win32 file:/// paths need a drive letter')
+    url_base += path[0:3] # file:// + /C:
+    path = path[3:] # /foo
+    return url_base, path
+
+
 def split(url, exclude_trailing_slash=True):
     """Split a URL into its parent directory and a child directory.
 
@@ -320,12 +345,10 @@
 
     if sys.platform == 'win32' and url.startswith('file:///'):
         # Strip off the drive letter
+        # url_base is currently file://
         # path is currently /C:/foo
-        if path[2:3] not in ':|' or path[3:4] not in '\\/':
-            raise errors.InvalidURL(url, 
-                'win32 file:/// paths need a drive letter')
-        url_base += path[0:3] # file:// + /C:
-        path = path[3:] # /foo
+        url_base, path = _win32_extract_drive_letter(url_base, path)
+        # now it should be file:///C: and /foo
 
     if exclude_trailing_slash and len(path) > 1 and path.endswith('/'):
         path = path[:-1]
@@ -333,6 +356,14 @@
     return url_base + head, tail
 
 
+def _win32_strip_local_trailing_slash(url):
+    """Strip slashes after the drive letter"""
+    if len(url) > WIN32_MIN_ABS_FILEURL_LENGTH:
+        return url[:-1]
+    else:
+        return url
+
+
 def strip_trailing_slash(url):
     """Strip trailing slash, except for root paths.
 
@@ -352,18 +383,13 @@
         file:///foo/      => file:///foo
         # This is unique on win32 platforms, and is the only URL
         # format which does it differently.
-        file:///C|/       => file:///C|/
+        file:///c|/       => file:///c:/
     """
     if not url.endswith('/'):
         # Nothing to do
         return url
     if sys.platform == 'win32' and url.startswith('file:///'):
-        # This gets handled specially, because the 'top-level'
-        # of a win32 path is actually the drive letter
-        if len(url) > MIN_ABS_FILEURL_LENGTH:
-            return url[:-1]
-        else:
-            return url
+        return _win32_strip_local_trailing_slash(url)
 
     scheme_loc, first_path_slash = _find_scheme_and_separator(url)
     if scheme_loc is None:

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-06-20 03:57:11 +0000
+++ bzrlib/versionedfile.py	2006-06-20 05:32:16 +0000
@@ -255,6 +255,14 @@
         return ''.join(self.get_lines(version_id))
     get_string = get_text
 
+    def get_texts(self, version_ids):
+        """Return the texts of listed versions as a list of strings.
+
+        Raises RevisionNotPresent if version is not present in
+        file history.
+        """
+        return [''.join(self.get_lines(v)) for v in version_ids]
+
     def get_lines(self, version_id):
         """Return version contents as a sequence of lines.
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-06-20 04:45:30 +0000
+++ bzrlib/workingtree.py	2006-06-20 05:32:16 +0000
@@ -59,7 +59,6 @@
 from bzrlib.errors import (BzrCheckError,
                            BzrError,
                            ConflictFormatError,
-                           DivergedBranches,
                            WeaveRevisionNotPresent,
                            NotBranchError,
                            NoSuchFile,
@@ -458,6 +457,20 @@
     def get_file_byname(self, filename):
         return file(self.abspath(filename), 'rb')
 
+    def get_parent_ids(self):
+        """See Tree.get_parent_ids.
+        
+        This implementation reads the pending merges list and last_revision
+        value and uses that to decide what the parents list should be.
+        """
+        last_rev = self.last_revision()
+        if last_rev is None:
+            parents = []
+        else:
+            parents = [last_rev]
+        other_parents = self.pending_merges()
+        return parents + other_parents
+
     def get_root_id(self):
         """Return the id of this trees root"""
         inv = self.read_working_inventory()
@@ -512,8 +525,10 @@
         # but with branch a kwarg now, passing in args as is results in the
         #message being used for the branch
         args = (DEPRECATED_PARAMETER, message, ) + args
-        Commit().commit(working_tree=self, revprops=revprops, *args, **kwargs)
+        committed_id = Commit().commit( working_tree=self, revprops=revprops,
+            *args, **kwargs)
         self._set_inventory(self.read_working_inventory())
+        return committed_id
 
     def id2abspath(self, file_id):
         return self.abspath(self.id2path(file_id))
@@ -1566,7 +1581,7 @@
         except NoSuchFile:
             raise errors.NoWorkingTree(base=transport.base)
         except KeyError:
-            raise errors.UnknownFormatError(format_string)
+            raise errors.UnknownFormatError(format=format_string)
 
     @classmethod
     def get_default_format(klass):

=== modified file 'doc/configuration.txt'
--- doc/configuration.txt	2006-03-14 16:22:52 +0000
+++ doc/configuration.txt	2006-06-16 15:59:24 +0000
@@ -2,8 +2,8 @@
 ==============================
 Each user gets a pair of configurations files in $HOME/.bazaar. The first
 one, named bazaar.conf, includes default configuration options. The other
-file, branches.conf, contains configuration information for specific
-branches.
+file, locations.conf, contains configuration information for specific
+branch locations.
 
 General Format
 ==============
@@ -27,7 +27,7 @@
 senstive. The default section provides for setting variables which can be
 overridden with the branch config file.
 
-For branches.conf, the variables from the section with the longest matching
+For locations.conf, the variables from the section with the longest matching
 section header are used to the exclusion of other potentially valid section
 headers. A section header uses the path for the branch as the section
 header. Some examples include::
@@ -54,7 +54,7 @@
 The main configuration file, $HOME/.bazaar/bazaar.conf, only allows one
 section called '''[DEFAULT]'''. This default section contains the default
 configuration options for all branches. The default section can be
-overriden by providing a branch specific section in branches.conf.
+overriden by providing a branch specific section in locations.conf.
 
 A typical bazaar.conf section often looks like the following::
 
@@ -64,7 +64,7 @@
     check_signatures  = check-available
     create_signatures = when-required
 
-$HOME/.bazaar/branches.conf allows one to specify overriding settings for a
+$HOME/.bazaar/locations.conf allows one to specify overriding settings for a
 specific branch. The format is almost identical to the default section in
 bazaar.conf with one significant change: The section header, instead of
 saying default, will be the path to a branch that you wish to override a
@@ -128,7 +128,7 @@
 
 recurse
 -------
-Only useful in branches.conf. Defines whether or not the configuration for
+Only useful in locations.conf. Defines whether or not the configuration for
 this section applies to subdirectories:
 
 true

=== modified file 'doc/setting_up_email.txt'
--- doc/setting_up_email.txt	2006-03-14 16:57:35 +0000
+++ doc/setting_up_email.txt	2006-06-16 15:59:24 +0000
@@ -20,7 +20,8 @@
 The order of precedence is
 
  1. If the `$BZREMAIL` env variable is set, use it.
- 2. If an email is set for your current branch in the `branches.conf` ini file.
+ 2. If an email is set for your current branch in the `locations.conf` ini
+    file.
  3. If an email is set in the `bazaar.conf` default ini file.
  4. If the `$EMAIL` env variable is set.
  5. Bazaar-NG will try to guess based on your username and the hostname.
@@ -49,7 +50,7 @@
 ===================================
 
 The second approach is to set email on a branch by branch basis by using the
-`$HOME/.bazaar/branches.conf` ini file, thusly::
+`$HOME/.bazaar/locations.conf` ini file, thusly::
 
     [/some/branch/location]
     email=Your Name <name@other-isp.com>

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-05-05 08:24:34 +0000
+++ doc/tutorial.txt	2006-06-17 19:01:15 +0000
@@ -126,7 +126,7 @@
  
  1. Override the previous setting on a
  branch by branch basis by creating a branch section in
- ``~/.bazaar/branches.conf`` by adding the following lines::
+ ``~/.bazaar/locations.conf`` by adding the following lines::
 
     [/the/directory/to/the/branch]
     email=Your Name <email@isp.com>
@@ -389,13 +389,12 @@
     % mkdir src
     % echo 'int main() {}' > src/simple.c
     % bzr add src
-    % bzr status
-    A       src/
-    ?       src/simple.c
-    % bzr add src/simple.c
-    % bzr status
-    A       src/
-    A       src/simple.c
+    added src
+    added src/simple.c
+    % bzr status
+    added:
+      src/
+      src/simple.c
 
 
 Deleting and removing files
@@ -415,9 +414,12 @@
     % bzr remove -v hello.txt
     ?       hello.txt
     % bzr status
-    ?       hello.txt
-    D       src/
-    D       src/simple.c
+    removed:
+      hello.txt
+      src/
+      src/simple.c
+    unknown:
+      hello.txt
 
 If you remove the wrong file by accident, you can use **bzr revert** to
 restore it.
@@ -497,5 +499,5 @@
 
   (The directory that must already exist)
 
-* The push plugin that comes with BzrTools
+* The rspush plugin that comes with BzrTools
 

=== added file 'profile_imports.py'
--- profile_imports.py	1970-01-01 00:00:00 +0000
+++ profile_imports.py	2006-06-19 14:40:19 +0000
@@ -0,0 +1,173 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Written by John Arbash Meinel <john@arbash-meinel.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A custom importer and regex compiler which logs time spent."""
+
+import sre
+import sys
+import time
+
+
+_parent_stack = []
+_total_stack = {}
+_info = {}
+_cur_id = 0
+
+
+def stack_add(name, frame_name, frame_lineno, scope_name=None):
+    """Start a new record on the stack"""
+    global _cur_id
+    _cur_id += 1
+    this_stack = (_cur_id, name)
+
+    if _parent_stack:
+        _total_stack[_parent_stack[-1]].append(this_stack)
+    _total_stack[this_stack] = []
+    _parent_stack.append(this_stack)
+    _info[this_stack] = [len(_parent_stack)-1, frame_name, frame_lineno, scope_name]
+
+    return this_stack
+
+
+def stack_finish(this, cost):
+    """Finish a given entry, and record its cost in time"""
+    global _parent_stack
+
+    assert _parent_stack[-1] == this, \
+        'import stack does not end with this %s: %s' % (this, _parent_stack)
+    _parent_stack.pop()
+    _info[this].append(cost)
+
+
+def log_stack_info(out_file, sorted=True, hide_fast=True):
+    # Find all of the roots with import = 0
+    out_file.write(' cum  inline name\t\t\t\t\t\tframe\n')
+    todo = [(value[-1], key) for key,value in _info.iteritems() if value[0] == 0]
+
+    if sorted:
+        todo.sort()
+
+    while todo:
+        cum_time, cur = todo.pop()
+        children = _total_stack[cur]
+
+        c_times = []
+
+        info = _info[cur]
+        if hide_fast and info[-1] < 0.0001:
+            continue
+
+        # Compute the module time by removing the children times
+        mod_time = info[-1]
+        for child in children:
+            c_info = _info[child]
+            mod_time -= c_info[-1]
+            c_times.append((c_info[-1], child))
+
+        # indent, cum_time, mod_time, name,
+        # scope_name, frame_name, frame_lineno
+        out_file.write('%5.1f %5.1f %s %-35s\t@ %s:%d\n'
+            % (info[-1]*1000., mod_time*1000., '+'*info[0], 
+               cur[1][:40], info[1], info[2]))
+
+        if sorted:
+            c_times.sort()
+        else:
+            c_times.reverse()
+        todo.extend(c_times)
+
+
+_real_import = __import__
+
+def timed_import(name, globals, locals, fromlist):
+    """Wrap around standard importer to log import time"""
+
+    scope_name = globals.get('__name__', None)
+    if scope_name is None:
+        scope_name = globals.get('__file__', None)
+    if scope_name is None:
+        scope_name = globals.keys()
+    else:
+        # Trim out paths before bzrlib
+        loc = scope_name.find('bzrlib')
+        if loc != -1:
+            scope_name = scope_name[loc:]
+        # For stdlib, trim out early paths
+        loc = scope_name.find('python2.4')
+        if loc != -1:
+            scope_name = scope_name[loc:]
+
+    # Figure out the frame that is doing the importing
+    frame = sys._getframe(1)
+    frame_name = frame.f_globals.get('__name__', '<unknown>')
+    extra = ''
+    cur_frame = 1
+    if frame_name.endswith('demandload'):
+        # If this was demandloaded, we have 3 frames to ignore
+        extra = ' (demandload)'
+        frame = sys._getframe(4)
+        cur_frame = 4
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
+    frame_lineno = frame.f_lineno
+
+    this = stack_add(name+extra, frame_name, frame_lineno, scope_name)
+
+    tstart = time.time()
+    try:
+        # Do the import
+        mod = _real_import(name, globals, locals, fromlist)
+    finally:
+        tload = time.time()-tstart
+        stack_finish(this, tload)
+
+    return mod
+
+
+_real_compile = sre._compile
+
+def timed_compile(*args, **kwargs):
+    """Log how long it takes to compile a regex"""
+
+    # And who is requesting this?
+    frame = sys._getframe(2)
+    frame_name = frame.f_globals.get('__name__', '<unknown>')
+    frame_lineno = frame.f_lineno
+
+    this = stack_add(repr(args[0]), frame_name, frame_lineno)
+
+    tstart = time.time()
+    try:
+        # Measure the compile time
+        comp = _real_compile(*args, **kwargs)
+    finally:
+        tcompile = time.time() - tstart
+        stack_finish(this, tcompile)
+
+    return comp
+
+
+def install():
+    """Install the hooks for measuring import and regex compile time."""
+    __builtins__['__import__'] = timed_import
+    sre._compile = timed_compile
+
+
+def uninstall():
+    """Remove the import and regex compile timing hooks."""
+    __builtins__['__import__'] = _real_import
+    sre._compile = _real_compile
+

